(*
Filename: BasicDefs.v

In this file we provide the formalization of basic Android concepts
that will be used in the rest of the specification.

*)
Require Export Maps.

Section Permission.

(*Permission identifiers*)
Parameter idPerm : Set.

(*Permission levels*)
Inductive permLevel : Set :=
| dangerous
| normal
| signature
| signatureOrSys.

(*Permission*)
Record Perm : Set := perm { idP: idPerm; pl: permLevel }.

(*Predefined permissions*)
Parameter isSystemPerm : Perm -> Prop.

End Permission.


Section Components.

(*Component identifier*)
Parameter idCmp : Set.

(*Content provider resource identifiers*)
Parameter uri : Set.
Axiom uri_eq : forall (id1 id2 : uri), {id1 = id2} + {id1 <> id2}.

(*Content provider resources*)
Parameter res: Set.

(*Mapping between resource ids and y content provider resources*)
Definition mapRes := mapping uri res.

(*Types of components*)
Record Activity : Set := activity { idA: idCmp }.
Record Service : Set := service { idS: idCmp }.
Record BroadReceiver : Set := broadreceiver { idB: idCmp }.
Record CProvider : Set := cprovider {idC: idCmp; map_res : mapping uri res}.

(*Generic components*)
Inductive Cmp : Set :=
  | cmpAct : Activity -> Cmp
  | cmpSrv : Service -> Cmp
  | cmpCP : CProvider -> Cmp
  | cmpBR : BroadReceiver -> Cmp.

End Components.


Section RunningComponents.

(*Component running instance identifier*)
Parameter idInst : Set.

(*Component running instance*)
Record iCmp : Set := icmp { idICmp : idInst }.

End RunningComponents.


Section Applications.

(*Application identifier*)
Parameter idApp : Set.

(*Application certificate*)
Parameter Cert: Set.

(*Application AndroidManifest file*)
Record Manifest : Type := mf { cmp : Cmp -> Prop;
                               use : Perm -> Prop;
                               usrP : Perm -> Prop;
			       exp : forall c:Cmp, cmp c -> Prop;	
                               appE : option Perm;
                               cmpE : forall c:Cmp, cmp c -> option Perm;
                               readE : forall cp:CProvider, cmp (cmpCP cp) -> option Perm;
                               writeE : forall cp:CProvider, cmp (cmpCP cp) -> option Perm;
                               grantU : forall cp:CProvider, cmp (cmpCP cp) -> Prop;
                               uriP : forall cp:CProvider, cmp (cmpCP cp) -> uri -> Prop }.

(*Application*)
Record App : Type := app { idAp : idApp;
                           cert : Cert;
                           manifest : Manifest;
                           (*We assume that the application statically
                             declares the resources that it will use*)
                           appRes : res -> Prop}.

End Applications.

Section Misc.
(*Type of operation (used for permission delegation) *)
Inductive PType :=
| Read
| Write
| Both.

(*Generic type for values*)
Parameter Val : Set.
End Misc.