(*
Filename: Exec.v

Formalization of the execution of an action on a valid state.
Definition of inductive predicates that represent the pre and post
conditions of each action.

*)
Require Export Semantics.

Section ActionExec.

(*Action pre-condition*)
Definition pre (act:Action) (s:State) : Prop :=
      match act with
      | install a => pre_install a s
      | uninstall a => pre_uninstall a s
      | start ic c => pre_start ic c s
      | stop ic => pre_stop ic s
      | call ic aCall => pre_call ic aCall s
      | read ic cp u => pre_read ic cp u s
      | write ic cp u v => pre_write ic cp u v s
      | grantT ic cp act u pt => pre_grantT ic cp act u pt s
      | grantP ic cp a u pt => pre_grantP ic cp a u pt s
      | revoke ic cp u pt => pre_revoke ic cp u pt s
      end.

(*Action post-condition*)
Definition post (act:Action) (s s':State) : Prop :=
      match act with
      | install a => post_install a s s'
      | uninstall a => post_uninstall a s s'
      | start ic c => post_start ic c s s'
      | stop ic => post_stop ic s s'
      | call ic aCall => post_call ic aCall s s'
      | read ic cp u => post_read ic cp u s s'
      | write ic cp u v => post_write ic cp u v s s'
      | grantT ic cp act u pt => post_grantT ic cp act u pt s s'
      | grantP ic cp a u pt => post_grantP ic cp a u pt s s'
      | revoke ic cp u pt => post_revoke ic cp u pt s s'
      end.

(*Action execution *)
Inductive exec (s: State) (act:Action) (s':State) : Prop :=
| Exec: validState s -> pre act s -> post act s s' -> exec s act s'.

End ActionExec.



