(*
Filename: Tactics.v

General tactics. 

*)
Require Import Exec.

(*validState destructors*)
Ltac destruct_valid_state H :=
    let HvalidState := fresh "H" in
    let H_allAppDifferent := fresh "H_allAppDifferent" in
    let H_allCmpDifferent := fresh "H_allCmpDifferent" in
    let H_notRepeatedCmps := fresh "H_notRepeatedCmps" in
    let H_notCPrunning := fresh "H_notCPrunning" in
    let H_usrPermsDefined := fresh "H_usrPermsDefined" in
    let H_existsAppnCPinDel := fresh "H_existsAppnCPinDel" in
    let H_delTmpRun := fresh "H_delTmpRun" in
    let H_cmpRunAppIns := fresh "H_cmpRunAppIns" in
    let H_notRepeatedIns := fresh "H_notRepeatedIns" in
    let H_resContAppInst := fresh "H_resContAppInst" in
    let H_resContOneVal := fresh "H_resContOneVal" in
      generalize H; intro HvalidState; 
       unfold validState in HvalidState;
        destruct HvalidState as
          (H_allAppDifferent,
          (H_allCmpDifferent,
          (H_notRepeatedCmps,
          (H_notCPrunning,
          (H_usrPermsDefined,
          (H_existsAppnCPinDel,
          (H_delTmpRun,
          (H_cmpRunAppIns,
          (H_notRepeatedIns,
          (H_resContAppInst))))))))));
           rename HvalidState into H_resContOneVal.

(*pre and post install destructors*)
Ltac destruct_pre_install H :=
    let Hpreinstall := fresh "H" in
    let H_new_app := fresh "H_new_app" in
    let H_not_repeated_cmp := fresh "H_not_repeated_cmp" in
    let H_new_cmp := fresh "H_new_cmp" in
    let H_auth_perms := fresh "H_auth_perms" in
      generalize H; intro Hpreinstall;
       unfold pre in Hpreinstall; unfold pre_install in Hpreinstall;
        destruct Hpreinstall as
          (H_new_app,
          (H_not_repeated_cmp,
          (H_new_cmp)));
	    rename Hpreinstall into H_auth_perms.

Ltac destruct_post_install H :=
    let Hpostinstall := fresh "H" in
    let H_addApp_1 := fresh "H_addApp_1" in
    let H_addApp_2 := fresh "H_addApp_2" in
    let H_addApp_3 := fresh "H_addApp_3" in
    let H_addRes_1 := fresh "H_addRes_1" in
    let H_addRes_2 := fresh "H_addRes_2" in
    let H_addRes_3 := fresh "H_addRes_3" in
    let H_addDefPerms_1 := fresh "H_addDefPerms_1" in
    let H_addDefPerms_2 := fresh "H_addDefPerms_2" in
    let H_addDefPerms_3 := fresh "H_addDefPerms_3" in
    let H_grantPPerms_1 := fresh "H_grantPPerms_1" in
    let H_grantPPerms_2 := fresh "H_grantPPerms_2" in
    let H_grantPPerms_3 := fresh "H_grantPPerms_3" in
    let H_sim_1:= fresh "H_sim_1" in
    let H_sim_2:= fresh "H_sim_2" in
    let H_sim_3:= fresh "H_sim_3" in
    let H_sim_4:= fresh "H_sim_4" in
      generalize H; intro Hpostinstall; 
       unfold post in Hpostinstall; unfold post_install in Hpostinstall;
        destruct Hpostinstall as
          (H_addApp_1,
          (H_addApp_2,
          (H_addApp_3,
          (H_addRes_1,
          (H_addRes_2,
          (H_addRes_3,
          (H_addDefPerms_1,
          (H_addDefPerms_2,
          (H_addDefPerms_3,
          (H_grantPPerms_1,
          (H_grantPPerms_2,
          (H_grantPPerms_3,
	  (H_sim_1,
	  (H_sim_2,
	  (H_sim_3)))))))))))))));
           rename Hpostinstall into H_sim_4.

(*pre and post-uninstall destructors*)
Ltac destruct_pre_uninstall H :=
    let Hpreuninstall := fresh "H" in
    let H_installed_app := fresh "H_installed_app" in
    let H_not_running := fresh "H_not_running" in
      generalize H; intro Hpreuninstall;
       unfold pre in Hpreuninstall; unfold pre_uninstall in Hpreuninstall;
        destruct Hpreuninstall as
          (H_installed_app);
	    rename Hpreuninstall into H_not_running.

Ltac destruct_post_uninstall H :=
    let Hpostuninstall := fresh "H" in
    let H_removeApp_1 := fresh "H_removeApp_1" in
    let H_removeApp_2 := fresh "H_removeApp_2" in
    let H_removeApp_3 := fresh "H_removeApp_3" in
    let H_revokePerms_1 := fresh "H_revokePerms_1" in
    let H_revokePerms_2 := fresh "H_revokePerms_2" in
    let H_revokePerms_3 := fresh "H_revokePerms_3" in
    let H_removeDefPerms_1 := fresh "H_removeDefPerms_1" in
    let H_removeDefPerms_2 := fresh "H_removeDefPerms_2" in
    let H_removeDefPerms_3 := fresh "H_removeDefPerms_3" in
    let H_removeRes_1 := fresh "H_removeRes_1" in
    let H_removeRes_2 := fresh "H_removeRes_2" in
    let H_removeRes_3 := fresh "H_removeRes_3" in
    let H_revokeOtherTPerms_1 := fresh "H_revokeOtherTPerms_1" in
    let H_revokeOtherTPerms_2 := fresh "H_revokeOtherTPerms_2" in
    let H_revokeOtherTPerms_3 := fresh "H_revokeOtherTPerms_3" in
    let H_revokeSOPerms_1:= fresh "H_revokeSOPerms_1" in
    let H_revokeSOPerms_2:= fresh "H_revokeSOPerms_2" in
    let H_revokeSOPerms_3:= fresh "H_revokeSOPerms_3" in
    let H_revokeSOPerms_4:= fresh "H_revokeSOPerms_4" in
    let H_sim_1:= fresh "H_sim_1" in
    let H_sim_2:= fresh "H_sim_2" in
      generalize H; intro Hpostuninstall; 
       unfold post in Hpostuninstall; unfold post_uninstall in Hpostuninstall;
        destruct Hpostuninstall as
          (H_removeApp_1,
          (H_removeApp_2,
          (H_removeApp_3,
          (H_revokePerms_1,
          (H_revokePerms_2,
          (H_revokePerms_3,
          (H_removeDefPerms_1,
          (H_removeDefPerms_2,
          (H_removeDefPerms_3,
          (H_removeRes_1,
          (H_removeRes_2,
          (H_removeRes_3,
          (H_revokeOtherTPerms_1,
          (H_revokeOtherTPerms_2,
          (H_revokeOtherTPerms_3,
	  (H_revokeSOPerms_1,
	  (H_revokeSOPerms_2,
	  (H_revokeSOPerms_3,
          (H_revokeSOPerms_4,
          (H_sim_1))))))))))))))))))));
           rename Hpostuninstall into H_sim_2.

(*pre and post-start destructors*)
Ltac destruct_pre_start H :=
    let Hprestart := fresh "H" in
    let H_cmpInstalled_c_s := fresh "H_cmpInstalled_c_s" in
    let H_notCP_c := fresh "H_notCP_c" in
    let H_exists_canStart := fresh "H_exists_canStart" in
     generalize H; intro Hprestart;
      unfold pre in Hprestart; unfold pre_start in Hprestart;
       destruct Hprestart as
          (H_cmpInstalled_c_s,
          (H_notCP_c));
	    rename Hprestart into H_exists_canStart.

Ltac destruct_post_start H :=
    let Hpoststart := fresh "H" in
    let H_runCmp := fresh "H_runCmp" in
    let H_sim_1 := fresh "H_sim_1" in
    let H_sim_2 := fresh "H_sim_2" in
    let H_sim_3 := fresh "H_sim_3" in
    let H_sim_4 := fresh "H_sim_4" in
    let H_sim_5 := fresh "H_sim_5" in
    let H_sim_6 := fresh "H_sim_6" in
    let H_sim_7 := fresh "H_sim_7" in
      generalize H; intro Hpoststart; 
       unfold post in Hpoststart; unfold post_start in Hpoststart;
        destruct Hpoststart as
          (H_runCmp,
          (H_sim_1,
          (H_sim_2,
          (H_sim_3,
          (H_sim_4,
          (H_sim_5,
          (H_sim_6)))))));
            rename Hpoststart into H_sim_7.

(* pre and post-stop destructors*)
Ltac destruct_pre_stop H :=
    let Hprestop := fresh "H" in
    let H_running := fresh "H_running" in
     generalize H; intro Hprestop; 
      unfold pre in Hprestop; unfold pre_stop in Hprestop;
       rename Hprestop into H_running.

Ltac destruct_post_stop H :=
    let Hpoststop := fresh "H" in
    let H_stopIns_1 := fresh "H_stopIns_1" in
    let H_stopIns_2 := fresh "H_stopIns_2" in
    let H_stopIns_3 := fresh "H_stopIns_3" in
    let H_revokeTPermsIns_1 := fresh "H_revokeTPermsIns_1" in
    let H_revokeTPermsIns_2 := fresh "H_revokeTPermsIns_2" in
    let H_revokeTPermsIns_3 := fresh "H_revokeTPermsIns_3" in
    let H_sim_1 := fresh "H_sim_1" in
    let H_sim_2 := fresh "H_sim_2" in
    let H_sim_3 := fresh "H_sim_3" in
    let H_sim_4 := fresh "H_sim_4" in
    let H_sim_5 := fresh "H_sim_5" in
    let H_sim_6 := fresh "H_sim_6" in
      generalize H; intro Hpoststop; 
       unfold post in Hpoststop; unfold post_stop in Hpoststop;
        destruct Hpoststop as
          (H_stopIns_1,
          (H_stopIns_2,
          (H_stopIns_3,
          (H_revokeTPermsIns_1,
          (H_revokeTPermsIns_2,
          (H_revokeTPermsIns_3,
          (H_sim_1,
          (H_sim_2,
          (H_sim_3,
          (H_sim_4,
          (H_sim_5)))))))))));
           rename Hpoststop into H_sim_6.

(*pre read destructor*)
Ltac destruct_pre_read H :=
    let Hpreread := fresh "H" in
    let H_cmpInstalled_cp_s := fresh "H_cmpInstalled_cp_s" in
    let H_existsRes := fresh "H_existsRes" in
    let H_exists_c_canRead := fresh "H_exists_c_canRead" in
      generalize H; intro Hpreread;
       unfold pre in Hpreread; unfold pre_read in Hpreread;
        destruct Hpreread as
          (H_cmpInstalled_cp_s,
          (H_existsRes));
	    rename Hpreread into H_exists_c_canRead.

(*pre- and post-write destructors*)
Ltac destruct_pre_write H :=
    let Hprewrite := fresh "H" in
    let H_cmpInstalled_cp_s := fresh "H_cmpInstalled_cp_s" in
    let H_existsRes := fresh "H_existsRes" in
    let H_exists_c_canWrite := fresh "H_exists_c_canWrite" in
      generalize H; intro Hprewrite;
       unfold pre in Hprewrite; unfold pre_write in Hprewrite;
        destruct Hprewrite as
          (H_cmpInstalled_cp_s,
          (H_existsRes));
	    rename Hprewrite into H_exists_c_canWrite.

Ltac destruct_post_write H :=
    let Hpostwrite := fresh "H" in
    let H_writeVal_1 := fresh "H_writeVal_1" in
    let H_writeVal_2 := fresh "H_writeVal_2" in
    let H_writeVal_3 := fresh "H_writeVal_3" in
    let H_writeVal_4 := fresh "H_writeVal_4" in
    let H_sim_1 := fresh "H_sim_1" in
    let H_sim_2 := fresh "H_sim_2" in
    let H_sim_3 := fresh "H_sim_3" in
    let H_sim_4 := fresh "H_sim_4" in
    let H_sim_5 := fresh "H_sim_5" in
    let H_sim_6 := fresh "H_sim_6" in
    let H_sim_7 := fresh "H_sim_7" in
      generalize H; intro Hpostwrite; 
       unfold post in Hpostwrite; unfold post_write in Hpostwrite;
        destruct Hpostwrite as
          (H_writeVal_1,
          (H_writeVal_2,
          (H_writeVal_3,
          (H_writeVal_4,
          (H_sim_1,
          (H_sim_2,
          (H_sim_3,
          (H_sim_4,
          (H_sim_5,
          (H_sim_6))))))))));
            rename Hpostwrite into H_sim_7.

(*pre and post grantT destructors*)
Ltac destruct_pre_grantT H :=
    let HpregrantT := fresh "H" in
    let H_cmpInstalled_cp_s := fresh "H_cmpInstalled_cp_s" in
    let H_canGrant := fresh "H_canGrant" in
    let H_existsRes := fresh "H_existsRes" in
    let H_cmpInstalled_act_s := fresh "H_cmpInstalled_act_s" in
    let H_exists_c_canStart := fresh "H_exists_c_canStart" in
      generalize H; intro HpregrantT;
       unfold pre in HpregrantT; unfold pre_grantT in HpregrantT;
        destruct HpregrantT as
          (H_cmpInstalled_cp_s,
          (H_canGrant,
          (H_existsRes,
          (H_cmpInstalled_act_s))));
	    rename HpregrantT into H_exists_c_canStart.

Ltac destruct_post_grantT H :=
    let HpostgrantT := fresh "H" in
    let H_runAndGrantT := fresh "H_runAndGrantT" in
    let H_sim_1 := fresh "H_sim_1" in
    let H_sim_2 := fresh "H_sim_2" in
    let H_sim_3 := fresh "H_sim_3" in
    let H_sim_4 := fresh "H_sim_4" in
    let H_sim_5 := fresh "H_sim_5" in
    let H_sim_6 := fresh "H_sim_6" in
      generalize H; intro HpostgrantT; 
       unfold post in HpostgrantT; unfold post_grantT in HpostgrantT;
        destruct HpostgrantT as
          (H_runAndGrantT,
          (H_sim_1,
          (H_sim_2,
          (H_sim_3,
          (H_sim_4,
          (H_sim_5))))));
            rename HpostgrantT into H_sim_6.

(*pre and post-grantP destructors*)
Ltac destruct_pre_grantP H :=
    let HpregrantP := fresh "H" in
    let H_cmpInstalled_cp_s := fresh "H_cmpInstalled_cp_s" in
    let H_canGrant := fresh "H_canGrant" in
    let H_existsRes := fresh "H_existsRes" in
    let H_appa := fresh "H_appa" in
    let H_exists_c := fresh "H_exists_c" in
      generalize H; intro HpregrantP;
       unfold pre in HpregrantP; unfold pre_grantP in HpregrantP;
        destruct HpregrantP as
          (H_cmpInstalled_cp_s,
          (H_canGrant,
          (H_existsRes,
          (H_appa))));
	    rename HpregrantP into H_exists_c.

Ltac destruct_post_grantP H :=
    let HpostgrantP := fresh "H" in
    let H_grantPPerm_1 := fresh "H_grantPPerm_1" in
    let H_grantPPerm_2 := fresh "H_grantPPerm_2" in
    let H_grantPPerm_3 := fresh "H_grantPPerm_3" in
    let H_sim_1 := fresh "H_sim_1" in
    let H_sim_2 := fresh "H_sim_2" in
    let H_sim_3 := fresh "H_sim_3" in
    let H_sim_4 := fresh "H_sim_4" in
    let H_sim_5 := fresh "H_sim_5" in
    let H_sim_6 := fresh "H_sim_6" in
    let H_sim_7 := fresh "H_sim_7" in
      generalize H; intro HpostgrantP; 
       unfold post in HpostgrantP; unfold post_grantP in HpostgrantP;
        destruct HpostgrantP as
          (H_grantPPerm_1,
          (H_grantPPerm_2,
          (H_grantPPerm_3,
          (H_sim_1,
          (H_sim_2,
          (H_sim_3,
          (H_sim_4,
          (H_sim_5,
          (H_sim_6)))))))));
            rename HpostgrantP into H_sim_7.

(*pre- y post-revoke destructors*)
Ltac destruct_pre_revoke H :=
    let Hprerevoke := fresh "H" in
    let H_cmpInstalled_cp_s := fresh "H_cmpInstalled_cp_s" in
    let H_existsRes := fresh "H_existsRes" in
    let H_exists_c := fresh "H_exists_c" in
      generalize H; intro Hprerevoke; 
       unfold pre in Hprerevoke; unfold pre_revoke in Hprerevoke;
        destruct Hprerevoke as
         (H_cmpInstalled_cp_s,
          (H_existsRes));
            rename Hprerevoke into H_exists_c.

Ltac destruct_post_revoke H :=
    let Hpostrevoke := fresh "H" in
    let H_revokeTPerm_1 := fresh "H_revokeTPerm_1" in
    let H_revokeTPerm_2 := fresh "H_revokeTPerm_2" in
    let H_revokeTPerm_3 := fresh "H_revokeTPerm_3" in
    let H_revokeTPerm_4 := fresh "H_revokeTPerm_4" in
    let H_revokePPerm_1 := fresh "H_revokePPerm_1" in
    let H_revokePPerm_2 := fresh "H_revokePPerm_2" in
    let H_revokePPerm_3 := fresh "H_revokePPerm_3" in
    let H_revokePPerm_4 := fresh "H_revokePPerm_4" in
    let H_sim_1 := fresh "H_sim_1" in
    let H_sim_2 := fresh "H_sim_2" in
    let H_sim_3 := fresh "H_sim_3" in
    let H_sim_4 := fresh "H_sim_4" in
    let H_sim_5 := fresh "H_sim_5" in
    let H_sim_6 := fresh "H_sim_6" in
      generalize H; intro Hpostrevoke; 
       unfold post in Hpostrevoke; unfold post_revoke in Hpostrevoke;
        destruct Hpostrevoke as
          (H_revokeTPerm_1,
          (H_revokeTPerm_2,
          (H_revokeTPerm_3,
          (H_revokeTPerm_4,
          (H_revokePPerm_1,
          (H_revokePPerm_2,
          (H_revokePPerm_3,
          (H_revokePPerm_4,
          (H_sim_1,
          (H_sim_2,
          (H_sim_3,
          (H_sim_4,
          (H_sim_5)))))))))))));
            rename Hpostrevoke into H_sim_6.