(* 
Filename: allAppDifferent.v

In this file we prove that the state resulting from the execution of
any operation of type Action preserves the property allAppDifferent.
For that, an auxiliary lemma is proved for each Action operation.

*)

Require Import Tactics.

Require Export Exec.

Section AuxLemmas.

Variables s s' : State.

Lemma proyAllAppDifferent : validState s -> allAppDifferent s.
Proof.
intros H_valid.
destruct_valid_state H_valid.
assumption.
Qed.

Lemma installPresAllAppDifferent : forall (a:App), exec s (install a) s' -> 
                                                         allAppDifferent s'.
Proof.
intros a H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyAllAppDifferent H_valid); intros H_allAppDiff.
unfold allAppDifferent in *.
intros a1 a2 H_a1 H_a2 H_idEq.
destruct_pre_install H_pre.
destruct_post_install H_post.
assert ((apps s) a1 \/ a1 = a) as H_dis1.
 apply H_addApp_2; assumption.
assert ((apps s) a2 \/ a2 = a) as H_dis2.
 apply H_addApp_2; assumption.
elim H_dis1.
 intros.
 elim H_dis2.
  intros H_app2.
  apply H_allAppDiff; assumption.

  intros H_a2a.
  assert (idAp a <> idAp a1) as H_aNeqa1.
   apply (H_new_app a1); assumption.
  rewrite H_a2a in H_idEq.
  symmetry in H_idEq.
  absurd (idAp a = idAp a1); assumption.

  intros H_a1a.
  elim H_dis2.
   intros H_app2.
   assert (idAp a <> idAp a2) as H_aNeqa2.
    apply (H_new_app a2); assumption.
    rewrite H_a1a in H_idEq.
    absurd (idAp a = idAp a2); assumption.

    intros H_a2a.
    rewrite <- H_a2a in H_a1a.
    assumption.
Qed.

Lemma uninstallPresAllAppDifferent : forall (a:App), exec s (uninstall a) s' -> 
                                                      allAppDifferent s'.
Proof.
intros a H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyAllAppDifferent H_valid); intros H_allAppDiff.
unfold allAppDifferent in *.
intros a1 a2 H_a1 H_a2 H_idEq.
destruct_pre_uninstall H_pre.
destruct_post_uninstall H_post.
assert(apps s a1) as H_appa1.
 apply H_removeApp_1; assumption.
assert(apps s a2) as H_appa2.
 apply H_removeApp_1; assumption.
apply H_allAppDiff; assumption.
Qed.

Lemma startPresAllAppDifferent : forall (ic:iCmp)(c:Cmp), exec s (start ic c) s' ->
                                                            allAppDifferent s'.
Proof.
intros ic c H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyAllAppDifferent H_valid); intros H_allAppDiff.
unfold allAppDifferent in *.
intros a1 a2 H_a1 H_a2 H_idEq.
destruct_pre_start H_pre.
destruct_post_start H_post.
rewrite H_sim_1 in H_allAppDiff.
apply H_allAppDiff; assumption.
Qed.

Lemma stopPresAllAppDifferent : forall (ic:iCmp), exec s (stop ic) s' ->
                                                    allAppDifferent s'.
Proof.
intros ic H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyAllAppDifferent H_valid); intros H_allAppDiff.
unfold allAppDifferent in *.
intros a1 a2 H_a1 H_a2 H_idEq.
destruct_pre_stop H_pre.
destruct_post_stop H_post.
rewrite H_sim_1 in H_allAppDiff.
apply H_allAppDiff; assumption.
Qed.

Lemma callPresAllAppDifferent : forall (ic:iCmp)(sac:SACall), exec s (call ic sac) s' ->
                                                                allAppDifferent s'.
Proof.
intros ic sac H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyAllAppDifferent H_valid); intros H_allAppDiff.
unfold allAppDifferent in *.
intros a1 a2 H_a1 H_a2 H_idEq.
unfold post in H_post.
unfold post_call in H_post.
apply H_allAppDiff.
rewrite H_post; assumption.
rewrite H_post; assumption.
assumption.
Qed.

Lemma readPresAllAppDifferent : forall (ic:iCmp)(cp:CProvider)(u:uri), exec s (read ic cp u) s' ->
                                                                         allAppDifferent s'.
Proof.
intros ic cp u H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyAllAppDifferent H_valid); intros H_allAppDiff.
unfold allAppDifferent in *.
intros a1 a2 H_a1 H_a2 H_idEq.
unfold post in H_post.
unfold post_read in H_post.
apply H_allAppDiff.
rewrite H_post; assumption.
rewrite H_post; assumption.
assumption.
Qed.

Lemma writePresAllAppDifferent : forall (ic:iCmp)(cp:CProvider)(u:uri)(v:Val), exec s (write ic cp u v) s' ->
                                                                                  allAppDifferent s'.
Proof.
intros ic cp u v H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyAllAppDifferent H_valid); intros H_allAppDiff.
unfold allAppDifferent in *.
intros a1 a2 H_a1 H_a2 H_idEq.
destruct_post_write H_post.
apply H_allAppDiff.
rewrite H_sim_1; assumption.
rewrite H_sim_1; assumption.
assumption.
Qed.

Lemma grantTPresAllAppDifferent : forall (ic:iCmp)(cp:CProvider)(act:Activity)(u:uri)(pt:PType), exec s (grantT ic cp act u pt) s' ->
                                                                                                     allAppDifferent s'.
Proof.
intros ic cp act u pt H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyAllAppDifferent H_valid); intros H_allAppDiff.
unfold allAppDifferent in *.
intros a1 a2 H_a1 H_a2 H_idEq.
destruct_post_grantT H_post.
apply H_allAppDiff.
rewrite H_sim_1; assumption.
rewrite H_sim_1; assumption.
assumption.
Qed.

Lemma grantPPresAllAppDifferent : forall (ic:iCmp)(cp:CProvider)(a:App)(u:uri)(pt:PType), exec s (grantP ic cp a u pt) s' ->
                                                                                            allAppDifferent s'.
Proof.
intros ic cp a u pt H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyAllAppDifferent H_valid); intros H_allAppDiff.
unfold allAppDifferent in *.
intros a1 a2 H_a1 H_a2 H_idEq.
destruct_post_grantP H_post.
apply H_allAppDiff.
rewrite H_sim_1; assumption.
rewrite H_sim_1; assumption.
assumption.
Qed.

Lemma revokePresAllAppDifferent : forall (ic:iCmp)(cp:CProvider)(u:uri)(pt:PType), exec s (revoke ic cp u pt) s' ->
                                                                                     allAppDifferent s'.
Proof.
intros ic cp u pt H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyAllAppDifferent H_valid); intros H_allAppDiff.
unfold allAppDifferent in *.
intros a1 a2 H_a1 H_a2 H_idEq.
destruct_post_revoke H_post.
apply H_allAppDiff.
rewrite H_sim_1; assumption.
rewrite H_sim_1; assumption.
assumption.
Qed.

End AuxLemmas.

Section allAppDifferentInv.

Theorem allAppDifferentInv : forall (a:Action)(s s':State), exec s a s' -> allAppDifferent s'.
Proof.
intros a s s'.
induction a.

exact (installPresAllAppDifferent s s' a).
exact (uninstallPresAllAppDifferent s s' a).
exact (startPresAllAppDifferent s s' i c).
exact (stopPresAllAppDifferent s s' i).
exact (callPresAllAppDifferent s s' i s0).
exact (readPresAllAppDifferent s s' i c u).
exact (writePresAllAppDifferent s s' i c u v).
exact (grantTPresAllAppDifferent s s' i c a u p).
exact (grantPPresAllAppDifferent s s' i c  a u p).
exact (revokePresAllAppDifferent s s' i c u p).
Qed.

End allAppDifferentInv.