(* 
Filename: allCmpDifferent.v

In this file we prove that the state resulting from the execution of
any operation of type Action preserves the property allCmpDifferent.
For that, an auxiliary lemma is proved for each Action operation.

*)
Require Import Tactics.

Require Export Exec.

Section AuxLemmas.

Variables s s' : State.

Lemma proyAllCmpDifferent : validState s -> allCmpDifferent s.
Proof.
intros H_valid.
destruct_valid_state H_valid.
assumption.
Qed.

Lemma installPresAllCmpDifferent : forall (a:App), exec s (install a) s' -> 
                                                    allCmpDifferent s'.
Proof.
intros a H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyAllCmpDifferent H_valid); intros H_allCmpDiff.
unfold allCmpDifferent in *.
intros c1 c2 a1 a2 H_inApp1 H_inApp2 H_app1_s' H_app2_s' H_cmpIdEq.
destruct_pre_install H_pre.
destruct_post_install H_post.
assert ((apps s) a1 \/ a1 = a) as H_dis1.
 apply H_addApp_2; assumption.
assert ((apps s) a2 \/ a2 = a) as H_dis2.
 apply H_addApp_2; assumption.
elim H_dis1.
 intros H_app1.
 elim H_dis2.
  intros H_app2.
  apply (H_allCmpDiff c1 c2 a1 a2); assumption.

  intros H_a2a.
  unfold cmpNotInState in H_new_cmp.
  rewrite <- H_a2a in H_new_cmp.
  assert (getCmpId c2 <> getCmpId c1) as H_c2Neqc1.
   apply (H_new_cmp c2 H_inApp2 c1 a1 H_app1 H_inApp1).
  symmetry in H_cmpIdEq.
  absurd (getCmpId c2 = getCmpId c1); assumption.

 intros H_a1a.
 elim H_dis2.
  intros H_app2.
  unfold cmpNotInState in H_new_cmp.
  rewrite <- H_a1a in H_new_cmp.
  assert (getCmpId c1 <> getCmpId c2) as H_c1Neqc1.
   apply (H_new_cmp c1 H_inApp1 c2 a2 H_app2 H_inApp2).
  absurd (getCmpId c1 = getCmpId c2); assumption.

  intros H_a2a.
  assert (getCmpId c1 <> getCmpId c2) as H_c1Neqc2.
   unfold cmpNotInState in H_new_cmp.
   rewrite H_a1a in H_inApp1.
   rewrite H_a2a in H_inApp2.
   apply H_not_repeated_cmp; assumption.
  absurd (getCmpId c1 = getCmpId c2); assumption.
Qed.

Lemma uninstallPresAllCmpDifferent : forall (a:App), exec s (uninstall a) s' -> 
                                                       allCmpDifferent s'.
Proof.
intros a H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyAllCmpDifferent H_valid); intros H_allCmpDiff.
unfold allCmpDifferent in *.
intros c1 c2 a1 a2 H_inApp1 H_inApp2 H_app1_s' H_app2_s' H_cmpIdEq.
destruct_pre_uninstall H_pre.
destruct_post_uninstall H_post.
assert(apps s a1) as H_appa1.
 apply H_removeApp_1; assumption.
assert(apps s a2) as H_appa2.
 apply H_removeApp_1; assumption.
apply (H_allCmpDiff c1 c2 a1 a2); assumption.
Qed.

Lemma startPresAllCmpDifferent : forall (ic:iCmp)(c:Cmp), exec s (start ic c) s' -> 
                                                             allCmpDifferent s'.
Proof.
intros ic c H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyAllCmpDifferent H_valid); intros H_allCmpDiff.
unfold allCmpDifferent in *.
intros c1 c2 a1 a2 H_inApp1 H_inApp2 H_app1_s' H_app2_s' H_cmpIdEq.
destruct_pre_start H_pre.
destruct_post_start H_post.
apply (H_allCmpDiff c1 c2 a1 a2).
assumption.
assumption.
rewrite H_sim_1; assumption.
rewrite H_sim_1; assumption.
assumption.
Qed.

Lemma stopPresAllCmpDifferent : forall (ic:iCmp), exec s (stop ic) s' ->
                                                      allCmpDifferent s'.
Proof.
intros ic H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyAllCmpDifferent H_valid); intros H_allCmpDiff.
unfold allCmpDifferent in *.
intros c1 c2 a1 a2 H_inApp1 H_inApp2 H_app1_s' H_app2_s' H_cmpIdEq.
destruct_pre_stop H_pre.
destruct_post_stop H_post.
apply (H_allCmpDiff c1 c2 a1 a2).
assumption.
assumption.
rewrite H_sim_1; assumption.
rewrite H_sim_1; assumption.
assumption.
Qed.

Lemma callPresAllCmpDifferent : forall (ic:iCmp)(sac:SACall), exec s (call ic sac) s' -> 
                                                                allCmpDifferent s'.
Proof.
intros ic sac H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyAllCmpDifferent H_valid); intros H_allCmpDiff.
unfold allCmpDifferent in *.
intros c1 c2 a1 a2 H_inApp1 H_inApp2 H_app1_s' H_app2_s' H_cmpIdEq.
unfold post in H_post.
unfold post_call in H_post.
apply (H_allCmpDiff c1 c2 a1 a2).
assumption.
assumption.
rewrite H_post; assumption.
rewrite H_post; assumption.
assumption.
Qed.

Lemma readPresAllCmpDifferent : forall (ic:iCmp)(cp:CProvider)(u:uri), exec s (read ic cp u) s' ->
                                                                           allCmpDifferent s'.
Proof.
intros ic cp u H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyAllCmpDifferent H_valid); intros H_allCmpDiff.
unfold allCmpDifferent in *.
intros c1 c2 a1 a2 H_inApp1 H_inApp2 H_app1_s' H_app2_s' H_cmpIdEq.
unfold post in H_post.
unfold post_read in H_post.
apply (H_allCmpDiff c1 c2 a1 a2).
assumption.
assumption.
rewrite H_post; assumption.
rewrite H_post; assumption.
assumption.
Qed.

Lemma writePresAllCmpDifferent : forall (ic:iCmp)(cp:CProvider)(u:uri)(v:Val), exec s (write ic cp u v) s' -> 
                                                                                    allCmpDifferent s'.
Proof.
intros ic cp u v H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyAllCmpDifferent H_valid); intros H_allCmpDiff.
unfold allCmpDifferent in *.
intros c1 c2 a1 a2 H_inApp1 H_inApp2 H_app1_s' H_app2_s' H_cmpIdEq.
destruct_post_write H_post.
apply (H_allCmpDiff c1 c2 a1 a2).
assumption.
assumption.
rewrite H_sim_1; assumption.
rewrite H_sim_1; assumption.
assumption.
Qed.

Lemma grantTPresAllCmpDifferent : forall (ic:iCmp)(cp:CProvider)(act:Activity)(u:uri)(pt:PType), exec s (grantT ic cp act u pt) s' ->
                                                                                                    allCmpDifferent s'.
Proof.
intros ic cp act u pt H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyAllCmpDifferent H_valid); intros H_allCmpDiff.
unfold allCmpDifferent in *.
intros c1 c2 a1 a2 H_inApp1 H_inApp2 H_app1_s' H_app2_s' H_cmpIdEq.
destruct_post_grantT H_post.
apply (H_allCmpDiff c1 c2 a1 a2).
assumption.
assumption.
rewrite H_sim_1; assumption.
rewrite H_sim_1; assumption.
assumption.
Qed.

Lemma grantPPresAllCmpDifferent : forall (ic:iCmp)(cp:CProvider)(a:App)(u:uri)(pt:PType), exec s (grantP ic cp a u pt) s' ->
                                                                                            allCmpDifferent s'.
Proof.
intros ic cp a u pt H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyAllCmpDifferent H_valid); intros H_allCmpDiff.
unfold allCmpDifferent in *.
intros c1 c2 a1 a2 H_inApp1 H_inApp2 H_app1_s' H_app2_s' H_cmpIdEq.
destruct_post_grantP H_post.
apply (H_allCmpDiff c1 c2 a1 a2).
assumption.
assumption.
rewrite H_sim_1; assumption.
rewrite H_sim_1; assumption.
assumption.
Qed.

Lemma revokePresAllCmpDifferent : forall (ic:iCmp)(cp:CProvider)(u:uri)(pt:PType), exec s (revoke ic cp u pt) s' ->
                                                                                        allCmpDifferent s'.
Proof.
intros ic cp u pt H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyAllCmpDifferent H_valid); intros H_allCmpDiff.
unfold allCmpDifferent in *.
intros c1 c2 a1 a2 H_inApp1 H_inApp2 H_app1_s' H_app2_s' H_cmpIdEq.
destruct_post_revoke H_post.
apply (H_allCmpDiff c1 c2 a1 a2).
assumption.
assumption.
rewrite H_sim_1; assumption.
rewrite H_sim_1; assumption.
assumption.
Qed.

End AuxLemmas.

Section allCmpDifferentInv.

Theorem allCmpDifferentInv: forall (a:Action)(s s':State), exec s a s' -> allCmpDifferent s'.
Proof.
intros a s s'.
induction a.

exact (installPresAllCmpDifferent s s' a).
exact (uninstallPresAllCmpDifferent s s' a).
exact (startPresAllCmpDifferent s s' i c).
exact (stopPresAllCmpDifferent s s' i).
exact (callPresAllCmpDifferent s s' i s0).
exact (readPresAllCmpDifferent s s' i c u).
exact (writePresAllCmpDifferent s s' i c u v).
exact (grantTPresAllCmpDifferent s s' i c a u p).
exact (grantPPresAllCmpDifferent s s' i c  a u p).
exact (revokePresAllCmpDifferent s s' i c u p).
Qed.

End allCmpDifferentInv.