(* 
Filename: canRedel.v

In this file we prove the possibility of permission re-delegation in
android.

*)
Require Import Exec.

Require Import Tactics.

(*PROPERTY: If ic or application a' have a delegated permission, it
can be re-delegated, either temporarily or permanently. *)
Lemma redelegation: forall (ic:iCmp)(c:Cmp)(a a':App)(act:Activity)(cp:CProvider)(u:uri)(pt:PType)(s:State),
                            validState s ->
                             apps s a ->
                              running s ic c ->
                               inApp c a ->
                                delTPerms s ic cp u pt \/
                                delPPerms s a cp u pt ->
                                 cmpInstalled (cmpCP cp) s ->
                                  cmpInstalled (cmpAct act) s ->
                                   apps s a' ->
                                    canStart c (cmpAct act) s ->
                                     canGrant cp u s -> 
                                      existsRes cp u s -> 
                                       pre_grantT ic cp act u pt s /\
                                       pre_grantP ic cp a' u pt s.
Proof.
intros ic c a a' act cp u pt s H_valid H_appa H_run_ic_c H_inApp_c_a H_dis H_cmpInstalled_cp_s H_cmpInstalled_act_s H_appa' H_canStart_c_act H_canGrant_u_s H_existsRes_u.
destruct_valid_state H_valid.
split.
unfold pre_grantT.
split.
assumption.
split.
assumption.
split.
assumption.
split.
assumption.
exists c.
split.
unfold cmpInstalled.
exists a; split; assumption.
split.
assumption.
split.
assumption.
case_eq pt.

intros H_ptRead.
right.
unfold delPerms.
intros a''. intros H_appa'' H_inApp_c_a''.
elim H_dis. clear H_dis. intros H_delTPerms_ic_u.
 right. left.
 exists ic. exists c.
 split.
 assumption.
 split.
 assumption.
 rewrite H_ptRead in H_delTPerms_ic_u.
 assumption.

 intros H_delPPerms_a_u.
 right. right. right.
 assert (a = a'') as H_aa''.
  unfold notRepeatedCmps in H_notRepeatedCmps.
  apply (H_notRepeatedCmps c a a''); assumption.
 rewrite <- H_aa''.
 rewrite H_ptRead in H_delPPerms_a_u.
 assumption.

intros H_ptWrite.
right.
unfold delPerms.
intros a''. intros H_appa'' H_inApp_ca''.
elim H_dis. clear H_dis. intros H_delTPerms_ic_u.
 right. left.
 exists ic. exists c.
 split.
 assumption.
 split.
 assumption.
 rewrite H_ptWrite in H_delTPerms_ic_u.
 assumption.

 intros H_delPPerms_a_u.
 right. right. right.
 assert (a = a'') as H_aa''.
  unfold notRepeatedCmps in H_notRepeatedCmps.
  apply (H_notRepeatedCmps c a a''); assumption.
 rewrite <- H_aa''.
 rewrite H_ptWrite in H_delPPerms_a_u.
 assumption.
 
 intros H_ptBoth.
 split.
 right.
 unfold delPerms.
 intros a''. intros H_appa'' H_inApp_c_a''.
 elim H_dis. clear H_dis. intros H_delTPerms_ic_u.
  left.
  exists ic. exists c.
  split.
  assumption.
  split.
  assumption.
  rewrite H_ptBoth in H_delTPerms_ic_u.
  assumption.

  intros H_delPPerms_a_u.
  right. right. left.
  assert (a = a'') as H_aa''.
   unfold notRepeatedCmps in H_notRepeatedCmps.
   apply (H_notRepeatedCmps c a a''); assumption.
  rewrite <- H_aa''.
  rewrite H_ptBoth in H_delPPerms_a_u.
  assumption.

  right.
  unfold delPerms.
  intros a''. intros H_appa'' H_inApp_ca''.
  elim H_dis. clear H_dis. intros H_delTPerms_ic_u.
   left.
   exists ic. exists c.
   split.
   assumption.
   split.
   assumption.
   rewrite H_ptBoth in H_delTPerms_ic_u.
   assumption.
    
   intros H_delPPerms_a_u.
   right. right. left.
   assert (a = a'') as H_aa''.
    unfold notRepeatedCmps in H_notRepeatedCmps.
    apply (H_notRepeatedCmps c a a''); assumption.
   rewrite <- H_aa''.
   rewrite H_ptBoth in H_delPPerms_a_u.
   assumption.

unfold pre_grantP.
split.
assumption.
split.
assumption.
split.
assumption.
split.
assumption.
exists c.
split.
unfold cmpInstalled.
exists a; split; assumption.
split.
assumption.
case_eq pt.

intros H_ptRead.
right.
unfold delPerms.
intros a''. intros H_appa'' H_inApp_c_a''.
elim H_dis. clear H_dis. intros H_delTPerms_ic_u.
 right. left.
 exists ic. exists c.
 split.
 assumption.
 split.
 assumption.
 rewrite H_ptRead in H_delTPerms_ic_u.
 assumption.

 intros H_delPPerms_a_u.
 right. right. right.
 assert (a = a'') as H_aa''.
  unfold notRepeatedCmps in H_notRepeatedCmps.
  apply (H_notRepeatedCmps c a a''); assumption.
 rewrite <- H_aa''.
 rewrite H_ptRead in H_delPPerms_a_u.
 assumption.

intros H_ptWrite.
right.
unfold delPerms.
intros a''. intros H_appa'' H_inApp_ca''.
elim H_dis. clear H_dis. intros H_delTPerms_ic_u.
 right. left.
 exists ic. exists c.
 split.
 assumption.
 split.
 assumption.
 rewrite H_ptWrite in H_delTPerms_ic_u.
 assumption.

 intros H_delPPerms_a_u.
 right. right. right.
 assert (a = a'') as H_aa''.
  unfold notRepeatedCmps in H_notRepeatedCmps.
  apply (H_notRepeatedCmps c a a''); assumption.
 rewrite <- H_aa''.
 rewrite H_ptWrite in H_delPPerms_a_u.
 assumption.
 
 intros H_ptBoth.
 split.
 right.
 unfold delPerms.
 intros a''. intros H_appa'' H_inApp_c_a''.
 elim H_dis. clear H_dis. intros H_delTPerms_ic_u.
  left.
  exists ic. exists c.
  split.
  assumption.
  split.
  assumption.
  rewrite H_ptBoth in H_delTPerms_ic_u.
  assumption.

  intros H_delPPerms_a_u.
  right. right. left.
  assert (a = a'') as H_aa''.
   unfold notRepeatedCmps in H_notRepeatedCmps.
   apply (H_notRepeatedCmps c a a''); assumption.
  rewrite <- H_aa''.
  rewrite H_ptBoth in H_delPPerms_a_u.
  assumption.

  right.
  unfold delPerms.
  intros a''. intros H_appa'' H_inApp_ca''.
  elim H_dis. clear H_dis. intros H_delTPerms_ic_u.
   left.
   exists ic. exists c.
   split.
   assumption.
   split.
   assumption.
   rewrite H_ptBoth in H_delTPerms_ic_u.
   assumption.
    
   intros H_delPPerms_a_u.
   right. right. left.
   assert (a = a'') as H_aa''.
    unfold notRepeatedCmps in H_notRepeatedCmps.
    apply (H_notRepeatedCmps c a a''); assumption.
   rewrite <- H_aa''.
   rewrite H_ptBoth in H_delPPerms_a_u.
   assumption.
Qed.