(* 
Filename: canRevoke.v

The three lemmas of this file give conditions that must hold when
revoking permissions.

*)
Require Import Exec.

Require Import Tactics.

(*PROPERTY: If ic revoked permissions to read a resource in cp pointed
by u, ic must itself be able to read that resource. *)
Lemma revokeCanRead : forall (ic:iCmp)(c:Cmp)(cp:CProvider)(u:uri)(s s':State),
                           running s ic c ->
                            exec s (revoke ic cp u Read) s'->
                             canRead c cp s.
Proof.
intros ic c cp u s s' H_run_ic_c H_exec.
inversion H_exec as [H_valid H_pre H_post].
destruct_valid_state H_valid.
destruct_pre_revoke H_pre.
elim H_exists_c. clear H_exists_c.
intros cc. intros H_conj.
elim H_conj. clear H_conj. intros H_cmpInstalled_cc_s H_conj.
elim H_conj. clear H_conj. intros H_run_ic_cc H_canRead_cc_cp.
assert (c = cc) as H_ccc.
 unfold notRepeatedIns in H_notRepeatedIns.
 apply (H_notRepeatedIns ic c cc); assumption.
rewrite H_ccc.
assumption.
Qed.

(*PROPERTY: If ic revoked permissions to write a resource in cp pointed
by u, ic must itself be able to write that resource. *)
Lemma revokeCanWrite : forall (ic:iCmp)(c:Cmp)(cp:CProvider)(u:uri)(s s':State),
                           running s ic c ->
                            exec s (revoke ic cp u Write) s'->
                             canWrite c cp s.
Proof.
intros ic c cp u s s' H_run_ic_c H_exec.
inversion H_exec as [H_valid H_pre H_post].
destruct_valid_state H_valid.
destruct_pre_revoke H_pre.
elim H_exists_c. clear H_exists_c.
intros cc. intros H_conj.
elim H_conj. clear H_conj. intros H_cmpInstalled_cc_s H_conj.
elim H_conj. clear H_conj. intros H_run_ic_cc H_canWrite_cc_cp.
assert (c = cc) as H_ccc.
 unfold notRepeatedIns in H_notRepeatedIns.
 apply (H_notRepeatedIns ic c cc); assumption.
rewrite H_ccc.
assumption.
Qed.

(*PROPERTY: If ic revoked permissions to read and write a resource in
cp pointed by u, ic must itself be able to read and write that resource. *)
Lemma revokeCanReadWrite : forall (ic:iCmp)(c:Cmp)(cp:CProvider)(u:uri)(s s':State),
                           running s ic c ->
                            exec s (revoke ic cp u Both) s'->
                             canRead c cp s /\ canWrite c cp s.
Proof.
intros ic c cp u s s' H_run_ic_c H_exec.
inversion H_exec as [H_valid H_pre H_post].
destruct_valid_state H_valid.
destruct_pre_revoke H_pre.
elim H_exists_c. clear H_exists_c.
intros cc. intros H_conj.
elim H_conj. clear H_conj. intros H_cmpInstalled_cc_s H_conj.
elim H_conj. clear H_conj. intros H_run_ic_cc H_conj.
elim H_conj. clear H_conj. intros H_canRead_cc_cp H_canWrite_cc_cp.
assert (c = cc) as H_ccc.
 unfold notRepeatedIns in H_notRepeatedIns.
 apply (H_notRepeatedIns ic c cc); assumption.
rewrite H_ccc.
split; assumption.
Qed.