(* 
Filename: canStart.v

The first lemma shows a sufficient condition to start a component. The
other two lemmas describe necessary conditions for the start of a
component in different scenarios.

*)
Require Import Exec.

Require Import Tactics.

(*PROPERTY: If c1 and c2 belong to the same application, c1 can start
c2. *)
Lemma StrSameApp : forall (ic1:iCmp)(c1 c2:Cmp)(a:App)(s:State),
                          validState s ->
                           running s ic1 c1 ->
                            apps s a ->
                             inApp c1 a ->
                               inApp c2 a ->
                                ~ isCProvider c2 ->
                                 pre_start ic1 c2 s.
Proof.
intros ic1 c1 c2 a s H_valid H_run_ic1_c1 H_appa H_inApp_c1_a H_inApp_c2_a H_notCP_c2.
destruct_valid_state H_valid.
unfold pre_start.
split.
exists a.
split; assumption.
split.
assumption.
exists c1.
split.
assumption.
split.
unfold cmpInstalled.
exists a.
split; assumption.
split.
apply (H_notCPrunning ic1 c1); assumption.
unfold canStart.
intros a1 a2 H_inApp_c1_a1 H_inApp_c2_a2 H_appa1 H_appa2.
assert (a1 = a) as H_a1a.
 unfold notRepeatedCmps in H_notRepeatedCmps.
 apply (H_notRepeatedCmps c1 a1 a); assumption.
assert (a2 = a) as H_a2a.
 apply (H_notRepeatedCmps c2 a2 a); assumption.
right.
rewrite H_a1a.
rewrite H_a2a.
reflexivity.
Qed.

(*PROPERTY: If c2 is not exported, and c1 belongs to another
application, then c1 cannot start c2. *)
Lemma notExpNotStred : forall (a:App)(c:Cmp)(H:inApp c a)(s:State),
                          validState s ->
                            apps s a ->
                              ~ exp (manifest a) c H ->
                                forall (a':App)(c':Cmp)(ic':iCmp),
                                  apps s a' -> 
                                   running s ic' c' ->
                                    inApp c' a' ->
                                     a <> a' ->
                                      ~ pre_start ic' c s.
Proof.
intros a c H_inApp_c_a s H_valid H_appa H_notExp_c a' c' ic' H_appa' H_run_ic'_c' H_inApp_c'_a' H_aNeqa'.
destruct_valid_state H_valid.
unfold not.
intros H_pre.
destruct_pre_start H_pre.
elim H_exists_canStart. clear H_exists_canStart.
intros c''. intros H_conj.
elim H_conj. clear H_conj. intros H_run_ic'_c'' H_conj.
elim H_conj. clear H_conj. intros H_cmpInstalled_c''_s H_conj.
elim H_conj. clear H_conj. intros H_notCP_cc H_canStart_c''_c_s.
assert (c' = c'') as H_c'c''.
 unfold notRepeatedIns in H_notRepeatedIns.
 apply (H_notRepeatedIns ic' c' c''); assumption.
unfold canStart in H_canStart_c''_c_s.
rewrite <- H_c'c'' in H_canStart_c''_c_s.
elim (H_canStart_c''_c_s a' a H_inApp_c'_a' H_inApp_c_a H_appa' H_appa).
 intros H_conj.
 elim H_conj. clear H_conj. intros H_exp H_forall.
 absurd (exp (manifest a) c H_inApp_c_a); assumption.

 intros H_a'a.
 symmetry in H_a'a.
 absurd (a = a'); assumption.
Qed.

(*PROPERTY: If c1 and c2 belong to different applications, and c2
requires a permission that c1 application does not have, c1 cannot
start c2. *)
Lemma StrEnfrcPermCmp : forall (c1 c2:Cmp)(a1 a2:App)(H1:inApp c1 a1)(H2:inApp c2 a2)(p:Perm)(s:State),
                      validState s ->
                       apps s a1 ->
                        apps s a2 ->
                         a1 <> a2 ->
                          cmpE (manifest a2) c2 H2 = Some p ->
                           ~ perms s a1 p ->
                            forall (ic1:iCmp), 
                             running s ic1 c1 ->
                              ~ pre_start ic1 c2 s.
Proof.
intros c1 c2 a1 a2 H_inApp_c1_a1 H_inApp_c2_a2 p s H_valid H_appa1 H_appa2 H_a1Neqa2 H_enf_c2 H_noPerm_a1_p ic1 H_run_ic1_c1.
destruct_valid_state H_valid.
unfold not.
intros H_pre.
destruct_pre_start H_pre.
elim H_exists_canStart. clear H_exists_canStart.
intros c1'. intros H_conj.
elim H_conj. clear H_conj. intros H_run_ic'_c1' H_conj.
elim H_conj. clear H_conj. intros H_cmpInstalled_c1'_s H_conj.
elim H_conj. clear H_conj. intros H_notCP_c1' H_canStart_c1'_c_s.
assert (c1 = c1') as H_c1c1'.
 unfold notRepeatedIns in H_notRepeatedIns.
 apply (H_notRepeatedIns ic1 c1 c1'); assumption.
rewrite <- H_c1c1' in H_canStart_c1'_c_s.
unfold canStart in H_canStart_c1'_c_s.
elim (H_canStart_c1'_c_s a1 a2 H_inApp_c1_a1 H_inApp_c2_a2 H_appa1 H_appa2).

 intros H_conj.
 elim H_conj. clear H_conj. intros H_exp_c2 H_forall.
 absurd (perms s a1 p).
  assumption.

  apply (H_forall p).
  left.
  assumption.

 intros H_a1a2.
 absurd (a1 = a2); assumption.
Qed.

(*PROPERTY: Assuming c1 and c2 belong to different applications, if c2
does not require permissions but its application does, and the
application of c1 does not have it, then c1 cannot start c2.*)
Lemma StrEnfrcPermApp : forall (c1 c2:Cmp)(a1 a2:App)(H1:inApp c1 a1)(H2:inApp c2 a2)(p:Perm)(s:State),
                      validState s ->
                       apps s a1 ->
                        apps s a2 ->
                         a1 <> a2 ->
                          cmpE (manifest a2) c2 H2 = None ->
                           appE (manifest a2) = Some p ->
                            ~ perms s a1 p ->
                             forall (ic1:iCmp), 
                              running s ic1 c1 ->
                               ~ pre_start ic1 c2 s.
Proof.
intros c1 c2 a1 a2 H_inApp_c1_a1 H_inApp_c2_a2 p s H_valid H_appa1 H_appa2 H_a1Neqa2 H_noEnf_c2 H_enf_a2 H_notPerm_a1_p ic1 H_run_ic1_c1.
destruct_valid_state H_valid.
unfold not.
intros H_pre.
destruct_pre_start H_pre.
elim H_exists_canStart. clear H_exists_canStart.
intros c1'. intros H_conj.
elim H_conj. clear H_conj. intros H_run_ic'_c1' H_conj.
elim H_conj. clear H_conj. intros H_cmpInstalled_c1'_s H_conj.
elim H_conj. clear H_conj. intros H_notCP_c1' H_canStart_c1'_c_s.
assert (c1 = c1') as H_c1c1'.
 unfold notRepeatedIns in H_notRepeatedIns.
 apply (H_notRepeatedIns ic1 c1 c1'); assumption.
rewrite <- H_c1c1' in H_canStart_c1'_c_s.
unfold canStart in H_canStart_c1'_c_s.
elim (H_canStart_c1'_c_s a1 a2 H_inApp_c1_a1 H_inApp_c2_a2 H_appa1 H_appa2).
 intros H_conj.
 elim H_conj. clear H_conj. intros H_exp_c2 H_forall.
 absurd (perms s a1 p).
  assumption.

  apply (H_forall p).
  right.
  split; assumption.

 intros H_a1a2.
 absurd (a1 = a2); assumption.
Qed.