(*
Filename: existsAppnCPinDel.v

In this file we prove that the state resulting from the execution of
actions preserves the property existsAppnCPinDel. We prove an
auxiliary lemma for each action.

*)
Require Import Tactics.

Require Export Exec.

Section AuxLemmas.

Variables s s' : State.

Lemma proyExistsAppnCPinDel : validState s -> existsAppnCPinDel s.
Proof.
intros H_valid.
destruct_valid_state H_valid.
assumption.
Qed.

Lemma installPresExistsAppnCPinDel : forall (a:App), exec s (install a) s' -> 
                                                    existsAppnCPinDel s'.
Proof.
intros a H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyExistsAppnCPinDel H_valid); intros H_existsCP.
unfold existsAppnCPinDel in *.
intros a' cp u pt H_a'delPPerms.
destruct_pre_install H_pre.
destruct_post_install H_post.
(*Permissions do not change from s to s'*)
rewrite <- H_sim_2 in H_a'delPPerms.
(*The property holds in s*)
assert (apps s a' /\ cmpInstalled (cmpCP cp) s /\ existsRes cp u s) as H_conj.
 apply (H_existsCP a' cp u pt); assumption.
elim H_conj. clear H_conj. intros H_appa' H_conj.
elim H_conj. clear H_conj. intros H_cpInstalled H_existsRes.
split.
apply H_addApp_1; assumption.
split.
unfold cmpInstalled in *.
elim H_cpInstalled.
intros aa. intros H_conj.
(*aa is the witness that shows cmpInstalled cp holds in s*)
exists aa.
split.
apply H_addApp_1.
elim H_conj; trivial.
elim H_conj; trivial.
unfold existsRes in *.
intros a'' H_appa''_s' H_inApp_cp_a''.
elim (H_addApp_2 a'' H_appa''_s').
 intros H_appa''_s.
 assert (exists r : res, map_apply uri_eq (map_res cp) u = Value uri r /\ (exists v : Val, resCont s a'' r v)) as H_exists.
  apply (H_existsRes a'' H_appa''_s H_inApp_cp_a'').
 elim H_exists.
 intros rr H_conj. clear H_exists.
 elim H_conj. clear H_conj. intros H_uMapsr H_existsVal.
 assert (exists v : Val, resCont s' a'' rr v) as H_val.
  elim H_existsVal.
  intros vv. intros H_resCont.
  exists vv.
  apply H_addRes_1; assumption.
 exists rr.
 split; assumption.

 intros H_aa''.
 rewrite H_aa'' in H_inApp_cp_a''.
 assert (cmpNotInState (cmpCP cp) s) as H_cmpNotInState.
  apply H_new_cmp.
  assumption.
 unfold cmpNotInState in H_cmpNotInState.
 unfold cmpInstalled in H_cpInstalled.
 elim H_cpInstalled.
 intros aa. intros H_conj.
 elim H_conj. clear H_conj. intros H_appaa H_inApp_cp_aa.
 assert (getCmpId (cmpCP cp) <> getCmpId (cmpCP cp)) as H_cpNeqcp.
  apply (H_cmpNotInState (cmpCP cp) aa); assumption.
 assert False as H_false.
  apply H_cpNeqcp.
  reflexivity.
 elim H_false.
Qed.

Lemma uninstallPresExistsAppnCPinDel : forall (a:App), exec s (uninstall a) s' -> 
                                                         existsAppnCPinDel s'.
Proof.
intros a H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyExistsAppnCPinDel H_valid); intros H_existsCP.
unfold existsAppnCPinDel in *.
intros a' cp u pt H_a'delPPerms.
(*a' is an application that has delegated permissions in s' to perform
  operation pt on a resource u of cp *)
destruct_pre_uninstall H_pre.
destruct_post_uninstall H_post.
(*If a' has delegated permissions in s', it already had them in s *)
assert (delPPerms s a' cp u pt) as H_delPPerms_a'_u_s.
 apply H_revokeSOPerms_1; assumption.
(*Since a' has permissions in s, the property holds by hypothesis *)
assert (apps s a' /\ cmpInstalled (cmpCP cp) s /\ existsRes cp u s) as H_conj.
 apply (H_existsCP a' cp u pt); assumption.
elim H_conj. clear H_conj. intros H_appa' H_conj.
elim H_conj. clear H_conj. intros H_cmpInstalled_cp H_existsRes.

(*We have to show that a' is installed in s' *)
(*By hypothesis, either a' is installed in s' or it is the uninstalled
application *)
assert (apps s' a' \/ a' = a) as H_dis.
 apply H_removeApp_2; assumption.
elim H_dis; clear H_dis.
 (*Case 1: a' is installed in s'*)
 intros H_appa'_s'.
 split.
 assumption.
 split.
 (*We know that cp is installed in s, prove that it is also installed
   in s' *)
 unfold cmpInstalled in *.
 elim H_cmpInstalled_cp.
 intros aCP. intros H_conj.
 (*aCP is the application owner of cp in s. Show that aCP will be the
   owner of cp in s' *)
 exists aCP.
 elim H_conj.
 intros H_appaCP H_inApp_cp_aCP.
 split.
 
 (*Either aCP is installed in s', or aCP was the uninstalled application *)
 elim (H_removeApp_2 aCP H_appaCP).
  (*Case 1.1: aCP is installed in s'*)
  trivial.
  (*Case 1.2: aCP was uninstalled (absurd) *)
  intros H_aCPa.
  (*The uninstalled application is the owner of cp *)
  rewrite H_aCPa in H_inApp_cp_aCP.
  (*By uninstall post-condition, there cannot be any delegation on cp
    from aCP. But a' has delegations on cp. Abusrd *)
  absurd (delPPerms s' a' cp u pt).
   apply H_revokeSOPerms_4.
   assumption.
   assumption.
 assumption.
 (*cp resource identified by u exists in s. Show that it also exists
   in s' *)
 unfold existsRes.
 intros a1 H_appa1_s' H_inApp_cp_a1.
 (*a1 is the owner of cp in s *)
 unfold existsRes in H_existsRes.
 (*By hypothesis, there is a resource en a1 associated to u by cp *)
 assert (exists r : res, map_apply uri_eq (map_res cp) u = Value uri r /\ (exists v : Val, resCont s a1 r v)) as H_conj.
  apply (H_existsRes a1).
  apply H_removeApp_1; assumption.
  assumption.
 elim H_conj. clear H_conj.
 intros rr. intros H_conj.
 (*rr is the resource*)
 elim H_conj. clear H_conj. intros H_ur H_exists_v.
 (*That same resource rr is associated to u in s' *)
 exists rr.
 split.
 assumption.
 elim H_exists_v.
 intros vv. intros H_resCont_rr_vv.
 (*vv is the value of resource rr of application a1 in s. In s' the
   resource has the same value *)
 exists vv.

 (*Either the resource rr exists in s', with the same values as in s,
   or it belongs to the uninstalled application *)
 elim (H_removeRes_2 a1 rr vv H_resCont_rr_vv).
  (*Case 1.1: Resource exists in s', with the same values*)
  trivial.
  (*Case 1.2: Resource was deleted during uninstallation *)
  intros H_a1a.
  (*Since a1 was owner of cp, cp belongs to the uninstalled
    application *)
  rewrite H_a1a in H_inApp_cp_a1.
  (* Uninstall post-condition implies that ther cannot be any
    delegation on cp of aCP. But a' has such delecations. Absurd.*)
  absurd (delPPerms s' a' cp u pt).
   apply H_revokeSOPerms_4.
   assumption.
   assumption.
 (*Case 2: a' is the uninstalled application *)
 intros H_a'a.
 rewrite <- H_a'a in H_revokeSOPerms_3.
 (*By hypothesis, the uninstalled application cannot have delegated
   permissions. But a' has delegated permissions. Absurd. *)
 absurd (delPPerms s' a' cp u pt).
  apply H_revokeSOPerms_3.
  assumption.
Qed.

Lemma startPresExistsAppnCPinDel : forall (ic:iCmp)(c:Cmp), exec s (start ic c) s' ->
                                                               existsAppnCPinDel s'.
Proof.
intros ic c H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyExistsAppnCPinDel H_valid); intros H_existsCP.
unfold existsAppnCPinDel in *.
intros a cp u pt H_a'delPPerms.
destruct_pre_start H_pre.
destruct_post_start H_post.
rewrite <- H_sim_4 in H_a'delPPerms.
assert (apps s a /\ cmpInstalled (cmpCP cp) s /\ existsRes cp u s) as H_conj.
 apply (H_existsCP a cp u pt); assumption.
unfold cmpInstalled in H_conj.
unfold existsRes in H_conj.
rewrite H_sim_1 in H_conj.
rewrite H_sim_6 in H_conj.
assumption.
Qed.

Lemma stopPresExistsAppnCPinDel : forall (ic:iCmp), exec s (stop ic) s' ->
                                                      existsAppnCPinDel s'.
Proof.
intros ic H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyExistsAppnCPinDel H_valid); intros H_existsCP.
unfold existsAppnCPinDel in *.
intros a cp u pt H_a'delPPerms.
destruct_pre_stop H_pre.
destruct_post_stop H_post.
rewrite <- H_sim_4 in H_a'delPPerms.
assert (apps s a /\ cmpInstalled (cmpCP cp) s /\ existsRes cp u s) as H_conj.
 apply (H_existsCP a cp u pt); assumption.
unfold cmpInstalled in H_conj.
unfold existsRes in H_conj.
rewrite H_sim_1 in H_conj.
rewrite H_sim_5 in H_conj.
assumption.
Qed.

Lemma callPresExistsAppnCPinDel : forall (ic:iCmp)(sac:SACall), exec s (call ic sac) s' ->
                                                                  existsAppnCPinDel s'.
Proof.
intros ic sac H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyExistsAppnCPinDel H_valid); intros H_existsCP.
unfold existsAppnCPinDel in *.
intros a cp u pt H_a'delPPerms.
rewrite <- H_post in H_a'delPPerms.
assert (apps s a /\ cmpInstalled (cmpCP cp) s /\ existsRes cp u s) as H_conj.
 apply (H_existsCP a cp u pt); assumption.
rewrite H_post in H_conj.
assumption.
Qed.

Lemma readPresExistsAppnCPinDel : forall (ic:iCmp)(cp:CProvider)(u:uri), exec s (read ic cp u) s' ->
                                                                          existsAppnCPinDel s'.
Proof.
intros ic cp u H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyExistsAppnCPinDel H_valid); intros H_existsCP.
unfold existsAppnCPinDel in *.
intros a cp' u' pt H_a'delPPerms.
rewrite <- H_post in H_a'delPPerms.
assert (apps s a /\ cmpInstalled (cmpCP cp') s /\ existsRes cp' u' s) as H_conj.
 apply (H_existsCP a cp' u' pt); assumption.
rewrite H_post in H_conj.
assumption.
Qed.

Lemma writePresExistsAppnCPinDel : forall (ic:iCmp)(cp:CProvider)(u:uri)(v:Val), exec s (write ic cp u v) s' ->
                                                                                     existsAppnCPinDel s'.
Proof.
intros ic cp u v H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyExistsAppnCPinDel H_valid); intros H_existsCP.
unfold existsAppnCPinDel in *.
intros a cp' u' pt H_a'delPPerms.
destruct_post_write H_post.
assert (apps s a /\ cmpInstalled (cmpCP cp') s /\ existsRes cp' u' s) as H_conj.
 apply (H_existsCP a cp' u' pt).
 rewrite H_sim_5.
 assumption.
elim H_conj. clear H_conj. intros H_appa H_conj.
elim H_conj. clear H_conj. intros H_cmpInstalled_cp'_s H_existsRes_u'_s.
split.
rewrite <- H_sim_1; assumption.
split.
unfold cmpInstalled in H_cmpInstalled_cp'_s.
rewrite H_sim_1 in H_cmpInstalled_cp'_s; assumption.
unfold existsRes in H_existsRes_u'_s.
unfold existsRes.
intros a' H_appa' H_inApp_cp'_a'.
assert (exists r : res,
        map_apply uri_eq (map_res cp') u' = Value uri r /\
        (exists v : Val, resCont s a' r v)) as H_exists.
 apply H_existsRes_u'_s.
 rewrite H_sim_1; assumption.
 assumption.
elim H_exists. clear H_exists.
intros rr. intros H_conj.
elim H_conj. clear H_conj. intros H_ur H_exists.
exists rr.
split.
assumption.
elim H_exists.
intros vv. intros H_resCont_rr_vv.
elim (H_writeVal_1 a' rr vv H_resCont_rr_vv).
 intros H_resCont_rr_vv_s'.
 exists vv.
 assumption.

 intros H_conj.
 exists v.
 elim H_conj. clear H_conj. intros H_inApp_cp_a' H_conj.
 elim H_conj. clear H_conj. intros H_appa'_s H_urr.
 apply (H_writeVal_3 a' rr H_inApp_cp_a' H_appa'_s).
 assumption.
Qed.

Lemma grantTPresExistsAppnCPinDel : forall (ic:iCmp)(cp:CProvider)(act:Activity)(u:uri)(pt:PType), exec s (grantT ic cp act u pt) s' ->
                                                                                                     existsAppnCPinDel s'.
Proof.
intros ic cp act u pt H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyExistsAppnCPinDel H_valid); intros H_existsCP.
unfold existsAppnCPinDel in *.
intros a cp' u' pt' H_a'delPPerms.
destruct_pre_grantT H_pre.
destruct_post_grantT H_post.
rewrite <- H_sim_4 in H_a'delPPerms.
assert (apps s a /\ cmpInstalled (cmpCP cp') s /\ existsRes cp' u' s) as H_conj.
 apply (H_existsCP a cp' u' pt'); assumption.
unfold cmpInstalled in H_conj.
unfold existsRes in H_conj.
rewrite H_sim_1 in H_conj.
rewrite H_sim_5 in H_conj.
assumption.
Qed.

Lemma grantPPresExistsAppnCPinDel : forall (ic:iCmp)(cp:CProvider)(a:App)(u:uri)(pt:PType), exec s (grantP ic cp a u pt) s' ->
                                                             existsAppnCPinDel s'.
Proof.
intros ic cp a u pt H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyExistsAppnCPinDel H_valid); intros H_existsCP.
unfold existsAppnCPinDel in *.
intros a' cp' u' pt' H_a'delPPerms.
(* a' is an application that has delegated permissions to perfrom
  action pt0 on resource u' of content provider cp' *)
destruct_pre_grantP H_pre.
destruct_post_grantP H_post.
unfold cmpInstalled.
rewrite <- H_sim_1.
unfold existsRes.
rewrite <- H_sim_6.
(*Either a' already had the permissions in s, or they were delegated
  by the execution of this action *)
elim (H_grantPPerm_2 a' cp' u' pt' H_a'delPPerms).
 (*Case 1: a' already had the permissions in s*)
 intros H_delPPerms_a'_u'_s.
 rewrite <- H_sim_1.
 (*By hypothesis, the property holds in s *)
  apply (H_existsCP a' cp' u' pt'); assumption.
 (*Case 2: The permission was delegated by the execution of this action *)
 intros H_conj.
 elim H_conj. clear H_conj. intros H_aa' H_conj.
 elim H_conj. clear H_conj. intros H_cpcp' H_conj.
 elim H_conj. clear H_conj. intros H_uu' H_ptpt'. 
 (*We use the precondition of the executed action. *)
 rewrite H_aa' in *.
 rewrite H_cpcp' in *.
 rewrite H_uu' in *.
 rewrite H_ptpt' in *.
 split; try assumption.
 rewrite <- H_sim_1.
 split; assumption.
Qed.

Lemma revokePresExistsAppnCPinDel : forall (ic:iCmp)(cp:CProvider)(u:uri)(pt:PType), exec s (revoke ic cp u pt) s' ->
                                                                                       existsAppnCPinDel s'.
Proof.
intros ic cp u pt H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyExistsAppnCPinDel H_valid); intros H_existsCP.
unfold existsAppnCPinDel in *.
intros a cp' u' pt' H_adelPPerms.
destruct_post_revoke H_post.
(*If a has delegated permissions in s', it already had them in s *)
assert (delPPerms s a cp' u' pt') as H_delPPerms_a_u'_s.
 apply H_revokePPerm_1; assumption.
(*Applications and resources do not change from s to s' *)
unfold cmpInstalled.
unfold existsRes.
rewrite <- H_sim_1.
rewrite <- H_sim_5.
apply (H_existsCP a cp' u' pt'); assumption.
Qed.

End AuxLemmas.

Section existsAppnCPinDelInv.

Theorem existsAppnCPinDelInv : forall (a:Action)(s s':State), exec s a s' -> existsAppnCPinDel s'.
Proof.
intros a s s'.
induction a.

exact (installPresExistsAppnCPinDel s s' a).
exact (uninstallPresExistsAppnCPinDel s s' a).
exact (startPresExistsAppnCPinDel s s' i c).
exact (stopPresExistsAppnCPinDel s s' i).
exact (callPresExistsAppnCPinDel s s' i s0).
exact (readPresExistsAppnCPinDel s s' i c u).
exact (writePresExistsAppnCPinDel s s' i c u v).
exact (grantTPresExistsAppnCPinDel s s' i c a u p).
exact (grantPPresExistsAppnCPinDel s s' i c  a u p).
exact (revokePresExistsAppnCPinDel s s' i c u p).
Qed.

End existsAppnCPinDelInv.