(*
Filename: :notRepeatedCmps.v

Proof that the execution of actions preserve the notRepeatedCmps
property.

*)
Require Import Tactics.

Require Export Exec.

Section AuxLemmas.

Variables s s' : State.

Lemma proyNotRepeatedCmps : validState s -> notRepeatedCmps s.
Proof.
intros H_valid.
destruct_valid_state H_valid.
assumption.
Qed.

Lemma installPresNotRepeatedCmps : forall (a:App), exec s (install a) s' -> 
                                                    notRepeatedCmps s'.
Proof.
intros a H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyNotRepeatedCmps H_valid); intros H_notRepCmps.
unfold notRepeatedCmps in *.
intros c a1 a2 H_app1_s' H_app2_s' H_inApp1 H_inApp2.
destruct_pre_install H_pre.
destruct_post_install H_post.
assert ((apps s) a1 \/ a1 = a) as H_dis1.
 apply H_addApp_2; assumption.
assert ((apps s) a2 \/ a2 = a) as H_dis2.
 apply H_addApp_2; assumption.
elim H_dis1.
 intros H_app1.
 elim H_dis2.
  intros H_app2.
  apply (H_notRepCmps c); assumption.

  intros H_a2a.
  unfold cmpNotInState in H_new_cmp.
  assert (getCmpId c <> getCmpId c) as H_cNeqc.
   rewrite H_a2a in H_inApp2.
   exact (H_new_cmp c H_inApp2 c a1 H_app1 H_inApp1).
   assert (False) as H_false.
    apply H_cNeqc.
    reflexivity.
   elim H_false.
 
 intros H_a1a.
 elim H_dis2.
  intros H_app2.
  unfold cmpNotInState in H_new_cmp.
  assert (getCmpId c <> getCmpId c) as H_cNeqc.
   rewrite H_a1a in H_inApp1.
   exact (H_new_cmp c H_inApp1 c a2 H_app2 H_inApp2).
  assert (False) as H_false.
   apply H_cNeqc.
   reflexivity.
  elim H_false.

  intros H_a2a.
  rewrite <- H_a2a in H_a1a.
  assumption.
Qed.

Lemma uninstallPresNotRepeatedCmps : forall (a:App), exec s (uninstall a) s' -> 
					  	        notRepeatedCmps s'.
Proof.

intros a H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyNotRepeatedCmps H_valid); intros H_notRepCmps.
unfold notRepeatedCmps in *.
intros c a1 a2 H_app1_s' H_app2_s' H_inApp1 H_inApp2.
destruct_pre_uninstall H_pre.
destruct_post_uninstall H_post.
assert(apps s a1) as H_appa1.
 apply H_removeApp_1; assumption.
assert(apps s a2) as H_appa2.
 apply H_removeApp_1; assumption.
apply (H_notRepCmps c a1 a2); assumption.
Qed.

Lemma startPresNotRepeatedCmps : forall (ic:iCmp)(c:Cmp), exec s (start ic c) s' -> 
                                                            notRepeatedCmps s'.
Proof.
intros ic c H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyNotRepeatedCmps H_valid); intros H_notRepCmps.
unfold notRepeatedCmps in *.
intros c' a1 a2 H_app1_s' H_app2_s' H_inApp1 H_inApp2.
destruct_pre_start H_pre.
destruct_post_start H_post.
apply (H_notRepCmps c' a1 a2).
rewrite H_sim_1; assumption.
rewrite H_sim_1; assumption.
assumption.
assumption.
Qed.

Lemma stopPresNotRepeatedCmps : forall (ic:iCmp), exec s (stop ic) s' -> 
                                         notRepeatedCmps s'.
Proof.
intros ic H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyNotRepeatedCmps H_valid); intros H_notRepCmps.
unfold notRepeatedCmps in *.
intros c a1 a2 H_app1_s' H_app2_s' H_inApp1 H_inApp2.
destruct_pre_stop H_pre.
destruct_post_stop H_post.
apply (H_notRepCmps c a1 a2).
rewrite H_sim_1; assumption.
rewrite H_sim_1; assumption.
assumption.
assumption.
Qed.

Lemma callPresNotRepeatedCmps : forall (ic:iCmp)(sac:SACall), exec s (call ic sac) s' ->
                                                                 notRepeatedCmps s'.
Proof.
intros ic sac H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyNotRepeatedCmps H_valid); intros H_notRepCmps.
unfold notRepeatedCmps in *.
intros c a1 a2 H_app1_s' H_app2_s' H_inApp1 H_inApp2.
apply (H_notRepCmps c a1 a2).
rewrite H_post; assumption.
rewrite H_post; assumption.
assumption.
assumption.
Qed.

Lemma readPresNotRepeatedCmps : forall (ic:iCmp)(cp:CProvider)(u:uri), exec s (read ic cp u) s' ->
     								          notRepeatedCmps s'.
Proof.
intros ic cp u H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyNotRepeatedCmps H_valid); intros H_notRepCmps.
unfold notRepeatedCmps in *.
intros c a1 a2 H_app1_s' H_app2_s' H_inApp1 H_inApp2.
apply (H_notRepCmps c a1 a2).
rewrite H_post; assumption.
rewrite H_post; assumption.
assumption.
assumption.
Qed.

Lemma writePresNotRepeatedCmps : forall (ic:iCmp)(cp:CProvider)(u:uri)(v:Val), exec s (write ic cp u v) s' ->
                                                                                 notRepeatedCmps s'.
Proof.
intros ic cp u v H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyNotRepeatedCmps H_valid); intros H_notRepCmps.
unfold notRepeatedCmps in *.
intros c a1 a2 H_app1_s' H_app2_s' H_inApp1 H_inApp2.
destruct_post_write H_post.
apply (H_notRepCmps c a1 a2).
rewrite H_sim_1; assumption.
rewrite H_sim_1; assumption.
assumption.
assumption.
Qed.

Lemma grantTPresNotRepeatedCmps : forall (ic:iCmp)(cp:CProvider)(act:Activity)(u:uri)(pt:PType), exec s (grantT ic cp act u pt) s' ->
                                                                                                   notRepeatedCmps s'.
Proof.
intros ic cp act u pt H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyNotRepeatedCmps H_valid); intros H_notRepCmps.
unfold notRepeatedCmps in *.
intros c a1 a2 H_app1_s' H_app2_s' H_inApp1 H_inApp2.
destruct_pre_grantT H_pre.
destruct_post_grantT H_post.
apply (H_notRepCmps c a1 a2).
rewrite H_sim_1; assumption.
rewrite H_sim_1; assumption.
assumption.
assumption.
Qed.

Lemma grantPPresNotRepeatedCmps : forall (ic:iCmp)(cp:CProvider)(a:App)(u:uri)(pt:PType), exec s (grantP ic cp a u pt) s' ->
                                                                                               notRepeatedCmps s'.
Proof.
intros ic cp a u pt H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyNotRepeatedCmps H_valid); intros H_notRepCmps.
unfold notRepeatedCmps in *.
intros c a1 a2 H_app1_s' H_app2_s' H_inApp1 H_inApp2.
destruct_post_grantP H_post.
apply (H_notRepCmps c a1 a2).
rewrite H_sim_1; assumption.
rewrite H_sim_1; assumption.
assumption.
assumption.
Qed.

Lemma revokePresNotRepeatedCmps : forall (ic:iCmp)(cp:CProvider)(u:uri)(pt:PType), exec s (revoke ic cp u pt) s' ->
                                                                                     notRepeatedCmps s'.
Proof.
intros ic cp u pt H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyNotRepeatedCmps H_valid); intros H_notRepCmps.
unfold notRepeatedCmps in *.
intros c a1 a2 H_app1_s' H_app2_s' H_inApp1 H_inApp2.
destruct_post_revoke H_post.
apply (H_notRepCmps c a1 a2).
rewrite H_sim_1; assumption.
rewrite H_sim_1; assumption.
assumption.
assumption.
Qed.

End AuxLemmas.

Section notRepeatedInsInv.

Theorem notRepeatedCmpsInv: forall (a:Action)(s s':State), exec s a s' -> notRepeatedCmps s'.
Proof.
intros a s s'.
induction a.

exact (installPresNotRepeatedCmps s s' a).
exact (uninstallPresNotRepeatedCmps s s' a).
exact (startPresNotRepeatedCmps s s' i c).
exact (stopPresNotRepeatedCmps s s' i).
exact (callPresNotRepeatedCmps s s' i s0).
exact (readPresNotRepeatedCmps s s' i c u).
exact (writePresNotRepeatedCmps s s' i c u v).
exact (grantTPresNotRepeatedCmps s s' i c a u p).
exact (grantPPresNotRepeatedCmps s s' i c  a u p).
exact (revokePresNotRepeatedCmps s s' i c u p).
Qed.

End notRepeatedInsInv.