(*
Filename: notRepeatedIns.v

Proof that the execution of actions preserve the notRepeatedIns
property.

*)
Require Import Tactics.

Require Export Exec.

Section AuxLemmas.

Variables s s' : State.

Lemma proyNotRepeatedIns : validState s -> notRepeatedIns s.
Proof.
intros H_valid.
destruct_valid_state H_valid.
assumption.
Qed.

Lemma installPresNotRepeatedIns : forall (a:App), exec s (install a) s' -> 
                                                    notRepeatedIns s'.
Proof.
intros a H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyNotRepeatedIns H_valid); intros H_notRepeatedIns.
unfold notRepeatedIns in *.
intros ic c1 c2 H_run_ic_c1_s' H_run_ic_c2_s'.
destruct_pre_install H_pre.
destruct_post_install H_post.
apply (H_notRepeatedIns ic c1 c2); (rewrite H_sim_1; assumption).
Qed.

Lemma uninstallPresNotRepeatedIns : forall (a:App), exec s (uninstall a) s' -> 
                                                      notRepeatedIns s'.
Proof.
intros a H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyNotRepeatedIns H_valid); intros H_notRepeatedIns.
unfold notRepeatedIns in *.
intros ic c1 c2 H_run_ic_c1_s' H_run_ic_c2_s'.
destruct_pre_uninstall H_pre.
destruct_post_uninstall H_post.
apply (H_notRepeatedIns ic c1 c2); (rewrite H_sim_1; assumption).
Qed.

Lemma startPresNotRepeatedIns : forall (ic:iCmp)(c:Cmp), exec s (start ic c) s' ->
                                                           notRepeatedIns s'.
Proof.
intros ic c H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyNotRepeatedIns H_valid); intros H_notRepeatedIns.
unfold notRepeatedIns in *.
intros ic' c1 c2 H_run_ic_c1_s' H_run_ic_c2_s'.
destruct_pre_start H_pre.
destruct_post_start H_post.
elim H_runCmp. clear H_runCmp. intros ic1 H_conj.
elim H_conj. clear H_conj. intros H_insNotInState_ic1_s H_conj.
elim H_conj. clear H_conj. intros H_runCmp_1 H_conj.
elim H_conj. clear H_conj. intros H_runCmp_2 H_runCmp_3.
assert (running s ic' c1 \/ ic1 = ic' /\ c = c1) as H_dis1.
 apply H_runCmp_2; assumption.
assert (running s ic' c2 \/ ic1 = ic' /\ c = c2) as H_dis2.
 apply H_runCmp_2; assumption.
unfold insNotInState in H_insNotInState_ic1_s.
elim H_dis1.
 clear H_dis1.
 intros H_run_ic'_ic.
 elim H_dis2.
  clear H_dis2.
  intros.
  apply (H_notRepeatedIns ic'); assumption.

  clear H_dis2.
  intros H_conj.
  assert (idICmp ic1 <> idICmp ic1) as H_c1Neqc1.
   elim H_conj. clear H_conj. intros H_ic1ic' H_cc2.
   rewrite <- H_ic1ic' in H_run_ic'_ic.
   apply (H_insNotInState_ic1_s ic1 c1); assumption.
  assert (False) as H_false.
   apply H_c1Neqc1.
   reflexivity.
  elim H_false.

 clear H_dis1.
 intros H_conj.
 elim H_dis2.
  clear H_dis2.
  intros H_run_ic'_c2.
  elim H_conj. clear H_conj. intros H_ic1ic' H_cc1.
  rewrite <- H_ic1ic' in H_run_ic'_c2.
  assert (idICmp ic1 <> idICmp ic1) as H_ic1Neqic1.
   apply (H_insNotInState_ic1_s ic1 c2); assumption.
  assert (False) as H_false.
   apply H_ic1Neqic1.
   reflexivity.
  elim H_false.

  clear H_dis2.
  intros H_conj2.
  elim H_conj.
  elim H_conj2.
  intros H_ic1ic' H_cc2 _ H_cc1.
  rewrite H_cc1 in H_cc2.
  assumption.
Qed.
   
Lemma stopPresNotRepeatedIns : forall (ic:iCmp), exec s (stop ic) s' ->
                                                   notRepeatedIns s'.
Proof.
intros ic H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyNotRepeatedIns H_valid); intros H_notRepeatedIns.
unfold notRepeatedIns in *.
intros ic' c1 c2 H_run_ic_c1_s' H_run_ic_c2_s'.
destruct_pre_stop H_pre.
destruct_post_stop H_post.
assert (running s ic' c1) as H_run_ic'_c1.
 apply H_stopIns_1; assumption.
assert (running s ic' c2) as H_run_ic'_c2.
 apply H_stopIns_1; assumption.
apply (H_notRepeatedIns ic' c1 c2); assumption.
Qed.

Lemma callPresNotRepeatedIns : forall (ic:iCmp)(sac:SACall), exec s (call ic sac) s' ->
                                                              notRepeatedIns s'.
Proof.
intros ic sac H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyNotRepeatedIns H_valid); intros H_notRepeatedIns.
unfold notRepeatedIns in *.
intros ic' c1 c2 H_run_ic_c1_s' H_run_ic_c2_s'.
unfold post in H_post.
unfold post_call in H_post.
apply (H_notRepeatedIns ic' c1 c2); (rewrite H_post; assumption).
Qed.

Lemma readPresNotRepeatedIns : forall (ic:iCmp)(cp:CProvider)(u:uri), exec s (read ic cp u) s' ->
                                                                       notRepeatedIns s'.
Proof.
intros ic cp u H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyNotRepeatedIns H_valid); intros H_notRepeatedIns.
unfold notRepeatedIns in *.
intros ic' c1 c2 H_run_ic_c1_s' H_run_ic_c2_s'.
apply (H_notRepeatedIns ic' c1 c2); (rewrite H_post; assumption).
Qed.

Lemma writePresNotRepeatedIns : forall (ic:iCmp)(cp:CProvider)(u:uri)(v:Val), exec s (write ic cp u v) s' ->
                                                                                notRepeatedIns s'.
Proof.
intros ic cp u v H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyNotRepeatedIns H_valid); intros H_notRepeatedIns.
unfold notRepeatedIns in *.
intros ic' c1 c2 H_run_ic_c1_s' H_run_ic_c2_s'.
destruct_post_write H_post.
apply (H_notRepeatedIns ic' c1 c2); (rewrite H_sim_4; assumption).
Qed.

Lemma grantTPresNotRepeatedIns : forall (ic:iCmp)(cp:CProvider)(act:Activity)(u:uri)(pt:PType), exec s (grantT ic cp act u pt) s' ->
                                                                                               notRepeatedIns s'.
Proof.
intros ic cp act u pt H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyNotRepeatedIns H_valid); intros H_notRepeatedIns.
unfold notRepeatedIns in *.
intros ic' c1 c2 H_run_ic_c1_s' H_run_ic_c2_s'.
destruct_pre_grantT H_pre.
destruct_post_grantT H_post.
elim H_runAndGrantT. clear H_runAndGrantT.
intros ic1. intros H_conj.
elim H_conj. clear H_conj. intros H_insNotinState_icc_s H_conj.
elim H_conj. clear H_conj. intros H_runCmp_1 H_conj.
elim H_conj. clear H_conj. intros H_runCmp_2 H_conj.
elim H_conj. clear H_conj. intros H_runCmp_3 H_conj.
elim H_conj. clear H_conj. intros H_grantTPerm_1 H_conj.
elim H_conj. clear H_conj. intros H_grantTPerm_2 H_grantTPerm_3.
assert (running s ic' c1 \/ ic1 = ic' /\ cmpAct act = c1) as H_dis1.
 apply H_runCmp_2; assumption.
assert (running s ic' c2 \/ ic1 = ic' /\ cmpAct act = c2) as H_dis2.
 apply H_runCmp_2; assumption.
unfold insNotInState in H_insNotinState_icc_s.
elim H_dis1.
 clear H_dis1.
 intros.
 elim H_dis2.
  clear H_dis2.
  intros H_run_ic'_c2.
  apply (H_notRepeatedIns ic'); assumption.

  clear H_dis2.
  intros H_conj.
  elim H_conj. clear H_conj. intros H_ic1ic' H_actc2.
  assert (idICmp ic1 <> idICmp ic1) as H_ic1Neqic1.
   apply (H_insNotinState_icc_s ic1 c1).
   rewrite H_ic1ic'.
   assumption.
   assert (False) as H_false.
    apply H_ic1Neqic1.
    reflexivity.
   elim H_false.

 clear H_dis1.
 intros H_conj.
 elim H_conj. clear H_conj. intros H_ic1ic' H_actc1.
 elim H_dis2.
  clear H_dis2.
  intros H_run_ic'_c2.
  assert (idICmp ic1 <> idICmp ic1) as H_ic1Neqic1.
   apply (H_insNotinState_icc_s ic1 c2).
   rewrite H_ic1ic'.
   assumption.
  assert (False) as H_false.
   apply H_ic1Neqic1.
   reflexivity.
  elim H_false.

  intros H_conj. elim H_conj. intros _ H_actc2.
  rewrite H_actc1 in H_actc2.
  assumption.
Qed.

Lemma grantPPresNotRepeatedIns : forall (ic:iCmp)(cp:CProvider)(a:App)(u:uri)(pt:PType), exec s (grantP ic cp a u pt) s' ->
                                                                                            notRepeatedIns s'.
Proof.
intros ic cp a u pt H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyNotRepeatedIns H_valid); intros H_notRepeatedIns.
unfold notRepeatedIns in *.
intros ic' c1 c2 H_run_ic_c1_s' H_run_ic_c2_s'.
destruct_post_grantP H_post.
apply (H_notRepeatedIns ic' c1 c2); (rewrite H_sim_4; assumption).
Qed.

Lemma revokePresNotRepeatedIns : forall (ic:iCmp)(cp:CProvider)(u:uri)(pt:PType), exec s (revoke ic cp u pt) s' ->
                                                                                   notRepeatedIns s'.
Proof.
intros ic cp u pt H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyNotRepeatedIns H_valid); intros H_notRepeatedIns.
unfold notRepeatedIns in *.
intros ic' c1 c2 H_run_ic_c1_s' H_run_ic_c2_s'.
destruct_post_revoke H_post.
apply (H_notRepeatedIns ic' c1 c2); (rewrite H_sim_4; assumption).
Qed.

End AuxLemmas.

Section notRepeatedInsInv.

Theorem notRepeatedInsInv : forall (a:Action)(s s':State), exec s a s' -> notRepeatedIns s'.
Proof.
intros a s s'.
induction a.

exact (installPresNotRepeatedIns s s' a).
exact (uninstallPresNotRepeatedIns s s' a).
exact (startPresNotRepeatedIns s s' i c).
exact (stopPresNotRepeatedIns s s' i).
exact (callPresNotRepeatedIns s s' i s0).
exact (readPresNotRepeatedIns s s' i c u).
exact (writePresNotRepeatedIns s s' i c u v).
exact (grantTPresNotRepeatedIns s s' i c a u p).
exact (grantPPresNotRepeatedIns s s' i c  a u p).
exact (revokePresNotRepeatedIns s s' i c u p).
Qed.

End notRepeatedInsInv.