(*
Filename: privEscalation.v

We prove that Android is vulnerable to a type of privilege escalation.

*)
Require Import Exec.

Require Import Tactics.

Definition cmpCanCall (c:Cmp) (sac:SACall) (s:State) :=
cmpInstalled c s /\ 
  (forall (a:App)(p:Perm)(H:isSystemPerm p), (apps s) a ->
             inApp c a -> permSAC p H sac -> (perms s) a p). 

(*PROPERTY: If ic cannot perform system call sac, but initiates a
  component c' that can; then, after starting c', sac can be called
  through a running instance of c'. *)
Lemma privEscalationAPI : forall (c1 c2:Cmp)(sac:SACall)(ic1:iCmp)(s s': State),
                	   running s ic1 c1 ->
                             ~ pre_call ic1 sac s ->
                              ~ pre_call ic1 sac s' ->
                               cmpCanCall c2 sac s' ->
                                exec s (start ic1 c2) s' ->
                                 exists ic2:iCmp, running s' ic2 c2 /\
                                  pre_call ic2 sac s'.
Proof.
intros c1 c2 sac ic1 s s' H_run_ic1_c1 H_cantCall_ic1_sac_s H_cantCall_ic1_sac_s' H_cmpCanCall_c2_sac_s' H_exec.
inversion H_exec as [H_valid H_pre H_post].
destruct_post_start H_post.
elim H_runCmp. clear H_runCmp. intros ic2 H_conj.
elim H_conj. clear H_conj. intros H_insNotInState_ic2_s H_conj.
elim H_conj. clear H_conj. intros H_runCmp_1 H_conj.
elim H_conj. clear H_conj. intros H_runCmp_2 H_runCmp_3.
exists ic2.
split.
assumption.
unfold pre_call.
exists c2.
split.
assumption.
elim H_cmpCanCall_c2_sac_s'.
trivial.
Qed.

(*PROPERTY: If ic cannot start a component c'', but initiates a
  component c' that can; then c'' can be started through a running
  instance of c'. *)
Lemma privEscalationCmps : forall (c1 c2 c3:Cmp)(ic1:iCmp)(s s': State),
                         cmpInstalled c3 s ->
                          ~ isCProvider c3 ->
                	   running s ic1 c1 ->
                           ~ canStart c1 c3 s ->
                            ~ canStart c1 c3 s' ->
                             canStart c2 c3 s' ->
                              exec s (start ic1 c2) s' ->
                               exists ic2:iCmp, running s' ic2 c2 /\
                                 pre_start ic2 c3 s'.
Proof.
intros c1 c2 c3 ic1 s s' H_cmpInstalled_c3_s H_notCP_c3 H_run_ic1_c1 H_cantStart_c1_c3_s H_cantStart_c1_c3_s' H_canStart_c2_c3 H_exec.
inversion H_exec as [H_valid H_pre H_post].
destruct_pre_start H_pre.
destruct_post_start H_post.
elim H_runCmp. clear H_runCmp. intros ic2 H_conj.
elim H_conj. clear H_conj. intros H_insNotInState_ic2_s H_conj.
elim H_conj. clear H_conj. intros H_runCmp_1 H_conj.
elim H_conj. clear H_conj. intros H_runCmp_2 H_runCmp_3.
exists ic2.
split.
assumption.
unfold pre_start.
split.
unfold cmpInstalled.
rewrite <- H_sim_1.
assumption.
split.
assumption.
exists c2.
split.
assumption.
split.
unfold cmpInstalled.
rewrite <- H_sim_1.
assumption.
split; assumption.
Qed.