(*
Filename: resContAppInst.v

Proof that the execution of actions preserve the resContAppInst
property.

*)
Require Import Tactics.

Require Export Exec.

Section AuxLemmas.

Variables s s' : State.

Lemma proyResContAppInst : validState s -> resContAppInst s.
Proof.
intros H_valid.
destruct_valid_state H_valid.
assumption.
Qed.

Lemma installPresResContAppInst : forall (a:App), exec s (install a) s' -> 
                                                    resContAppInst s'.
Proof.
intros a H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyResContAppInst H_valid); intros H_resContAppInst.
unfold resContAppInst in *.
intros a' r v H_resCont_s'.
destruct_pre_install H_pre.
destruct_post_install H_post.

elim (H_addRes_2 a' r v H_resCont_s').
 intros H_resCont_s.
 apply H_addApp_1.
 apply (H_resContAppInst a' r v).
 assumption.
 
 intros H_conj.
 elim H_conj. clear H_conj. intros H_a'a H_conj.
 rewrite H_a'a.
 assumption.
Qed.

Lemma uninstallPresResContAppInst : forall (a:App), exec s (uninstall a) s' ->
                                                      resContAppInst s'.
Proof.
intros a H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyResContAppInst H_valid); intros H_resContAppInst.
unfold resContAppInst in *.
intros a' r v H_resCont_s'.
destruct_pre_uninstall H_pre.
destruct_post_uninstall H_post.
assert (apps s a') as H_appa'.
 apply (H_resContAppInst a' r v).
 apply H_removeRes_1; assumption.
elim (H_removeApp_2 a' H_appa').
 trivial.

 intros H_a'a.
 absurd (resCont s' a' r v).
  rewrite H_a'a.
  apply H_removeRes_3.

  assumption.
Qed.

Lemma startPresResContAppInst : forall (ic:iCmp)(c:Cmp), exec s (start ic c) s' ->
			                                    resContAppInst s'.
Proof.
intros ic c H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyResContAppInst H_valid); intros H_resContAppInst.
unfold resContAppInst in *.
intros a r v H_resCont_s'.
destruct_pre_start H_pre.
destruct_post_start H_post.
rewrite <- H_sim_1.
apply (H_resContAppInst a r v).
rewrite H_sim_6.
assumption.
Qed.

Lemma stopPresResContAppInst : forall (ic:iCmp), exec s (stop ic) s' ->
                                                   resContAppInst s'.
Proof.
intros ic H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyResContAppInst H_valid); intros H_resContAppInst.
unfold resContAppInst in *.
intros a r v H_resCont_s'.
destruct_pre_stop H_pre.
destruct_post_stop H_post.
rewrite <- H_sim_1.
apply (H_resContAppInst a r v).
rewrite H_sim_5.
assumption.
Qed.

Lemma callPresResContAppInst : forall (ic:iCmp)(sac:SACall), exec s (call ic sac) s' ->
                                                               resContAppInst s'.
Proof.
intros ic sac H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyResContAppInst H_valid); intros H_resContAppInst.
unfold resContAppInst in *.
intros a r v H_resCont_s'.
unfold post in H_post.
unfold post_call in H_post.
rewrite <- H_post.
apply (H_resContAppInst a r v).
rewrite H_post.
assumption.
Qed.

Lemma readPresResContAppInst : forall (ic:iCmp)(cp:CProvider)(u:uri), exec s (read ic cp u) s' ->
                                                                        resContAppInst s'.
Proof.
intros ic cp u H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyResContAppInst H_valid); intros H_resContAppInst.
unfold resContAppInst in *.
intros a r v H_resCont_s'.
rewrite <- H_post.
apply (H_resContAppInst a r v).
rewrite H_post.
assumption.
Qed.

Lemma writePresResContAppInst : forall (ic:iCmp)(cp:CProvider)(u:uri)(v:Val), exec s (write ic cp u v) s' ->
                                                                                resContAppInst s'.
Proof.
intros ic cp u v H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyResContAppInst H_valid); intros H_resContAppInst.
unfold resContAppInst in *.
intros a r v0 H_resCont_s'.
destruct_post_write H_post.
rewrite <- H_sim_1.
elim (H_writeVal_2 a r v0 H_resCont_s').
 intros H_resCont_r_v0_s.
 apply (H_resContAppInst a r v0); assumption.

 intros H_conj.
 elim H_conj. clear H_conj. intros H_inApp_cp_a H_conj.
 elim H_conj. clear H_conj. intros H_appa H_conj.
 assumption.
Qed.

Lemma grantTPresResContAppInst : forall (ic:iCmp)(cp:CProvider)(act:Activity)(u:uri)(pt:PType), exec s (grantT ic cp act u pt) s' ->
                                                                                                  resContAppInst s'.
Proof.
intros ic cp act u pt H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyResContAppInst H_valid); intros H_resContAppInst.
unfold resContAppInst in *.
intros a r v H_resCont_s'.
destruct_post_grantT H_post.
rewrite <- H_sim_1.
apply (H_resContAppInst a r v).
rewrite H_sim_5.
assumption.
Qed.

Lemma grantPPresResContAppInst : forall (ic:iCmp)(cp:CProvider)(a:App)(u:uri)(pt:PType), exec s (grantP ic cp a u pt) s' ->
                                                                                           resContAppInst s'.
Proof.
intros ic cp a u pt H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyResContAppInst H_valid); intros H_resContAppInst.
unfold resContAppInst in *.
intros a' r v H_resCont_s'.
destruct_post_grantP H_post.
rewrite <- H_sim_1.
apply (H_resContAppInst a' r v).
rewrite H_sim_6.
assumption.
Qed.

Lemma revokePresResContAppInst : forall (ic:iCmp)(cp:CProvider)(u:uri)(pt:PType), exec s (revoke ic cp u pt) s' ->
                                       resContAppInst s'.
Proof.
intros ic cp u pt H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyResContAppInst H_valid); intros H_resContAppInst.
unfold resContAppInst in *.
intros a r v H_resCont_s'.
destruct_post_revoke H_post.
rewrite <- H_sim_1.
apply (H_resContAppInst a r v).
rewrite H_sim_5.
assumption.
Qed.

End AuxLemmas.

Section resContAppInstInv.

Lemma resContAppInstInv : forall (a:Action)(s s':State), exec s a s' -> resContAppInst s'.
Proof.
intros a s s'.
induction a.

exact (installPresResContAppInst s s' a).
exact (uninstallPresResContAppInst s s' a).
exact (startPresResContAppInst s s' i c).
exact (stopPresResContAppInst s s' i).
exact (callPresResContAppInst s s' i s0).
exact (readPresResContAppInst s s' i c u).
exact (writePresResContAppInst s s' i c u v).
exact (grantTPresResContAppInst s s' i c a u p).
exact (grantPPresResContAppInst s s' i c  a u p).
exact (revokePresResContAppInst s s' i c u p).
Qed.

End resContAppInstInv.