(*
Filename: resContOneVal.v

Proof that the execution of actions preserve the resContOneVal
property.

*)
Require Import Tactics.

Require Export Exec.

Section AuxLemmas.

Variables s s' : State.

Lemma proyResContOneVal : validState s -> resContOneVal s.
Proof.
intros H_valid.
destruct_valid_state H_valid.
assumption.
Qed.

Lemma proyResContAppInst : validState s -> resContAppInst s.
Proof.
intros H_valid.
destruct_valid_state H_valid.
assumption.
Qed.

Lemma installPresResContOneVal : forall (a:App), exec s (install a) s' -> 
                                                    resContOneVal s'.
Proof.
intros a H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyResContOneVal H_valid); intros H_resContOneVal.
generalize (proyResContAppInst H_valid); intros H_resContAppInst.
unfold resContOneVal in *.
intros a' r v1 v2 H_resCont_a'_v1_s' H_resCont_a'_v2_s'.
destruct_pre_install H_pre.
destruct_post_install H_post.
elim (H_addRes_2 a' r v1 H_resCont_a'_v1_s'); elim (H_addRes_2 a' r v2 H_resCont_a'_v2_s').
 intros H_resCont_a'_v2_s H_resCont_a'_v1_s.
 apply (H_resContOneVal a' r); assumption.

 intros H_conj H_resCont_a'_v1_s.
 assert (apps s a') as H_appa'.
  apply (H_resContAppInst a' r v1).
  assumption.
 assert False as H_false.
  unfold not in H_new_app.
  apply (H_new_app a').
  assumption.
  elim H_conj. clear H_conj. intros H_a'a H_conj.
  rewrite H_a'a.
  reflexivity.
 elim H_false.

 intros H_resCont_a'_v2_s H_conj.
 assert (apps s a') as H_apps'.
  apply (H_resContAppInst a' r v2).
  assumption.
 assert False as H_false.
  unfold not in H_new_app.
  apply (H_new_app a').
  assumption.
  elim H_conj. clear H_conj. intros H_a'a H_conj.
  rewrite H_a'a.
  reflexivity.
 elim H_false.

 intros H_conj.
 elim H_conj. clear H_conj. intros H_a'a H_conj.
 elim H_conj. clear H_conj. intros H_appRes H_initVal1 H_conj.
 elim H_conj. clear H_conj. intros H_a'a2. intros H_conj.
 elim H_conj. clear H_conj. intros H_appRes2 H_initVal2. 
 rewrite H_initVal1; rewrite H_initVal2.
 reflexivity.
Qed.

Lemma uninstallPresResContOneVal : forall (a:App), exec s (uninstall a) s' ->
                                                     resContOneVal s'.
Proof.
intros a H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyResContOneVal H_valid); intros H_resContOneVal.
generalize (proyResContAppInst H_valid); intros H_resContAppInst.
unfold resContOneVal in *.
intros a' r v1 v2 H_resCont_a'_v1_s' H_resCont_a'_v2_s'.
destruct_pre_uninstall H_pre.
destruct_post_uninstall H_post.
apply (H_resContOneVal a' r v1 v2).
apply (H_removeRes_1 a' r v1); assumption.
apply (H_removeRes_1 a' r v2); assumption.
Qed.

Lemma startPresResContOneVal : forall (ic:iCmp)(c:Cmp), exec s (start ic c) s' ->
                                                          resContOneVal s'.
Proof.
intros ic c H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyResContOneVal H_valid); intros H_resContOneVal.
unfold resContOneVal in *.
intros a r v1 v2 H_resCont_a'_v1_s' H_resCont_a'_v2_s'.
destruct_pre_start H_pre.
destruct_post_start H_post.
apply (H_resContOneVal a r v1 v2); rewrite H_sim_6; assumption.
Qed.

Lemma stopPresResContOneVal : forall (ic:iCmp), exec s (stop ic) s' ->
                                                  resContOneVal s'.
Proof.
intros ic H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyResContOneVal H_valid); intros H_resContOneVal.
unfold resContOneVal in *.
intros a r v1 v2 H_resCont_a'_v1_s' H_resCont_a'_v2_s'.
destruct_pre_stop H_pre.
destruct_post_stop H_post.
apply (H_resContOneVal a r v1 v2); rewrite H_sim_5; assumption.
Qed.

Lemma callPresResContOneVal : forall (ic:iCmp)(sac:SACall), exec s (call ic sac) s' ->
                                                             resContOneVal s'.
Proof.
intros ic sac H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyResContOneVal H_valid); intros H_resContOneVal.
unfold resContOneVal in *.
intros a r v1 v2 H_resCont_a'_v1_s' H_resCont_a'_v2_s'.
unfold post in H_post.
unfold post_call in H_post.
apply (H_resContOneVal a r v1 v2); rewrite H_post; assumption.
Qed.

Lemma readPresResContOneVal : forall (ic:iCmp)(cp:CProvider)(u:uri), exec s (read ic cp u) s' ->
                                                                         resContOneVal s'.
Proof.
intros ic cp u H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyResContOneVal H_valid); intros H_resContOneVal.
unfold resContOneVal in *.
intros a r v1 v2 H_resCont_a'_v1_s' H_resCont_a'_v2_s'.
apply (H_resContOneVal a r v1 v2); rewrite H_post; assumption.
Qed.

Lemma writePresResContOneVal : forall (ic:iCmp)(cp:CProvider)(u:uri)(v:Val), exec s (write ic cp u v) s' -> 
                                                                               resContOneVal s'.
Proof.
intros ic cp u v H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyResContOneVal H_valid); intros H_resContOneVal.
unfold resContOneVal in *.
intros a r v1 v2 H_resCont_a'_v1_s' H_resCont_a'_v2_s'.
destruct_post_write H_post.
elim (H_writeVal_2 a r v1 H_resCont_a'_v1_s'); elim (H_writeVal_2 a r v2 H_resCont_a'_v2_s').
 intros H_resCont_r_v1_s H_resCont_r_v2_s.
 apply (H_resContOneVal a r); assumption.

 intros H_conj.
 elim H_conj. clear H_conj. intros H_inApp_cp_a H_conj.
 elim H_conj. clear H_conj. intros H_appa H_conj.
 elim H_conj. clear H_conj. intros H_ur H_v2v H_resCont_r_v1_s.
 elim (H_writeVal_4 a r v1 H_inApp_cp_a H_appa H_ur H_resCont_r_v1_s).
  intros H_notResCont_r_v1_s.
  absurd (resCont s' a r v1); assumption.
  
  intros H_v1v.
  rewrite H_v1v; rewrite H_v2v.
  reflexivity.
 
  intros H_resCont_r_v2_s H_conj.
  elim H_conj. clear H_conj. intros H_inApp_cp_a H_conj.
  elim H_conj. clear H_conj. intros H_appa H_conj.
  elim H_conj. clear H_conj. intros H_ur H_v1v.
  elim (H_writeVal_4 a r v2 H_inApp_cp_a H_appa H_ur H_resCont_r_v2_s).
   intros H_notResCont_r_v2_s.
   absurd (resCont s' a r v2); assumption.
  
  intros H_v2v.
  rewrite H_v1v; rewrite H_v2v.
  reflexivity.
 
  intros H_conj.
  elim H_conj. clear H_conj. intros H_inApp_cp_a H_conj.
  elim H_conj. clear H_conj. intros H_appa H_conj.
  elim H_conj. clear H_conj. intros H_ur H_v2v H_conj.
  elim H_conj. clear H_conj. intros _ H_conj.
  elim H_conj. clear H_conj. intros _ H_conj.
  elim H_conj. clear H_conj. intros _ H_v1v.
  rewrite H_v1v; rewrite H_v2v.
  reflexivity.
Qed.

Lemma grantTPresResContOneVal : forall (ic:iCmp)(cp:CProvider)(act:Activity)(u:uri)(pt:PType), exec s (grantT ic cp act u pt) s' ->
                                                                                                 resContOneVal s'.
Proof.
intros ic cp act u pt H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyResContOneVal H_valid); intros H_resContOneVal.
unfold resContOneVal in *.
intros a r v1 v2 H_resCont_a'_v1_s' H_resCont_a'_v2_s'.
destruct_post_grantT H_post.
apply (H_resContOneVal a r v1 v2); rewrite H_sim_5; assumption.
Qed.

Lemma grantPPresResContOneVal : forall (ic:iCmp)(cp:CProvider)(a:App)(u:uri)(pt:PType), exec s (grantP ic cp a u pt) s' ->
                                       resContOneVal s'.
Proof.
intros ic cp a u pt H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyResContOneVal H_valid); intros H_resContOneVal.
unfold resContOneVal in *.
intros a' r v1 v2 H_resCont_a'_v1_s' H_resCont_a'_v2_s'.
destruct_post_grantP H_post.
apply (H_resContOneVal a' r v1 v2); rewrite H_sim_6; assumption.
Qed.

Lemma revokePresResContOneVal : forall (ic:iCmp)(cp:CProvider)(u:uri)(pt:PType), exec s (revoke ic cp u pt) s' ->
                                                                                    resContOneVal s'.
Proof.
intros ic cp u pt H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyResContOneVal H_valid); intros H_resContOneVal.
unfold resContOneVal in *.
intros a r v1 v2 H_resCont_a'_v1_s' H_resCont_a'_v2_s'.
destruct_post_revoke H_post.
apply (H_resContOneVal a r v1 v2); rewrite H_sim_5; assumption.
Qed.

End AuxLemmas.

Section resContOneValInv.

Lemma resContOneValInv : forall (a:Action)(s s':State), exec s a s' ->
                                                         resContOneVal s'.
Proof.
intros a s s'.
induction a.

exact (installPresResContOneVal s s' a).
exact (uninstallPresResContOneVal s s' a).
exact (startPresResContOneVal s s' i c).
exact (stopPresResContOneVal s s' i).
exact (callPresResContOneVal s s' i s0).
exact (readPresResContOneVal s s' i c u).
exact (writePresResContOneVal s s' i c u v).
exact (grantTPresResContOneVal s s' i c a u p).
exact (grantPPresResContOneVal s s' i c  a u p).
exact (revokePresResContOneVal s s' i c u p).
Qed.

End resContOneValInv.