(*
Filename: rvkCoarseGrnd.v

This file contains one lemma that shows the impossibility of revoking
permissions in a granular way.

*)
Require Import Exec.

Require Import Tactics.

(*PROPERTY: If ic revoked permissions on resource u of cp, then all
  delegated permissions associated to u will be revoked. *)
Lemma revokeCoarseGrnd : forall (ic:iCmp)(cp:CProvider)(u:uri)(pt:PType)(s s':State),
                                 exec s (revoke ic cp u pt) s' ->
                                  (forall (ic':iCmp)(c':Cmp),
                                   running s ic' c' -> ~ delTPerms s' ic' cp u pt) /\
                                  (forall (a:App), 
                                   apps s a -> ~ delPPerms s' a cp u pt).
Proof.
intros ic cp u pt s s' H_exec.
inversion H_exec as [H_valid H_pre H_post].
destruct_post_revoke H_post.
split.
intros ic' c' H_run_ic'_c'.
case_eq pt.
intros H_ptRead.
assert (forall ic' : iCmp, ~ delTPerms s' ic' cp u pt) as H_notDelTPerms.
 apply H_revokeTPerm_4.
 rewrite H_ptRead.
 unfold not.
 intros H_ReadBoth.
 discriminate.
rewrite H_ptRead in H_notDelTPerms.
apply H_notDelTPerms.

intros H_ptWrite.
assert (forall ic' : iCmp, ~ delTPerms s' ic' cp u pt) as H_notDelTPerms.
 apply H_revokeTPerm_4.
 rewrite H_ptWrite.
 unfold not.
 intros H_WriteBoth.
 discriminate.
rewrite H_ptWrite in H_notDelTPerms.
apply H_notDelTPerms.

intros H_ptBoth.
assert (forall (ic':iCmp)(pt':PType), ~ delTPerms s' ic' cp u pt') as H_notDelTPerms.
 apply H_revokeTPerm_3.
 rewrite H_ptBoth.
 reflexivity.
apply H_notDelTPerms.

intros a H_appa.
case_eq pt.

intros H_ptRead.
assert (forall a' : App, ~ delPPerms s' a' cp u pt) as H_notDelPPerms.
 apply H_revokePPerm_4.
 rewrite H_ptRead.
 unfold not.
 intros H_ReadBoth.
 discriminate.
rewrite H_ptRead in H_notDelPPerms.
apply H_notDelPPerms.

intros H_ptWrite.
assert (forall a' : App, ~ delPPerms s' a' cp u pt) as H_notDelPPerms.
 apply H_revokePPerm_4.
 rewrite H_ptWrite.
 unfold not.
 intros H_WriteBoth.
 discriminate.
rewrite H_ptWrite in H_notDelPPerms.
apply H_notDelPPerms.

intros H_ptBoth.
assert (forall (a':App)(pt':PType), ~ delPPerms s' a' cp u pt') as H_notDelPPerms.
 apply H_revokePPerm_3.
 rewrite H_ptBoth.
 reflexivity.
apply H_notDelPPerms.
Qed.