(*
Filename: usrPermsDefined.v

Proof that the execution of actions preserve the usrPermsDefined
property.

*)
Require Import Tactics.

Require Export Exec.

Section AuxLemmas.

Variables s s' : State.

Lemma proyUsrPermsDefined : validState s -> usrPermsDefined s.
Proof.
intros H_valid.
destruct_valid_state H_valid.
assumption.
Qed.

Lemma installPresUsrPermsDefined : forall (a:App), exec s (install a) s' -> 
                                                    usrPermsDefined s'.
Proof.
intros a H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyUsrPermsDefined H_valid); intros H_usrPermsDef.
unfold usrPermsDefined in *.
intros a' p H_defPerm_a'.
(*p is a permission defined by application a' in s'. We have to show
  that a' is installed and that p is defined in its manifest.*)
destruct_pre_install H_pre.
destruct_post_install H_post.
(*Either the permission was defined by an application in s, or it is
defined by the application being installed*)
elim (H_addDefPerms_2 a' p H_defPerm_a').
 (*Case 1: a' defined p in s*)
 intros H_defPerm_s.
 (*Application a' is installed in s and p is defined in its manifest*)
 assert (apps s a' /\ (usrP (manifest a') p)) as H_aux.
  apply H_usrPermsDef; assumption.
 elim H_aux.
 split.
 apply H_addApp_1; assumption.
 assumption.
 (*Case 2: a' is the application being installed, and it defines
   permission p that is not in the system*)
 intros H_conj.
 elim H_conj. clear H_conj. intros H_aa' H_conj.
 elim H_conj. clear H_conj. intros H_permDefInApp H_conj.
 elim H_conj. clear H_conj. intros H_notSysPerm H_hyp.
 split.
 rewrite <- H_aa' in H_addApp_3.
 assumption.
 unfold permDefInApp in H_permDefInApp.
 rewrite <- H_aa' in H_permDefInApp.
 assumption.
Qed.

Lemma uninstallPresUsrPermsDefined : forall (a:App), exec s (uninstall a) s' -> 
                                                        usrPermsDefined s'.
Proof.
intros a H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyUsrPermsDefined H_valid); intros H_usrPermsDef.
unfold usrPermsDefined in *.
intros a' p H_defPerm_a'.
(*p is a permission defined by application a' in s'. We have to show
  that a' is installed and that p is defined in its manifest.*)
destruct_pre_uninstall H_pre.
destruct_post_uninstall H_post.
assert (defPerms s a' p) as H_defPerms_a'_p_s.
 apply H_removeDefPerms_1; assumption.
assert (apps s a' /\ usrP (manifest a') p) as H_conj.
 apply (H_usrPermsDef a'); assumption.
elim H_conj. clear H_conj. intros H_appa' H_usrP_a'_p.
split.
(*Either application a'is still installed in s', or it was
  uninstalled.*)
assert (apps s' a' \/ a' = a) as H_dis.
 apply H_removeApp_2; assumption.
elim H_dis; clear H_dis.
 (*Case 1: a' is still installed in s'*)
 trivial.
 (*Case 2: a' is uninstalled in s'. Absurd.*)
 intros H_a'a.
 rewrite <- H_a'a in H_removeDefPerms_3.
 absurd (defPerms s' a' p). 
 apply (H_removeDefPerms_3 p).
 assumption.
assumption.
Qed.

Lemma startPresUsrPermsDefined : forall (ic:iCmp)(c:Cmp), exec s (start ic c) s' -> 
                                                            usrPermsDefined s'.
Proof.
intros ic c H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyUsrPermsDefined H_valid); intros H_usrPermsDef.
unfold usrPermsDefined in *.
intros a p H_defPerm_a.
destruct_pre_start H_pre.
destruct_post_start H_post.
rewrite <- H_sim_1.
apply (H_usrPermsDef a p).
rewrite H_sim_3.
assumption.
Qed.

Lemma stopPresUsrPermsDefined : forall (ic:iCmp), exec s (stop ic) s' -> 
                                                   usrPermsDefined s'.
Proof. 
intros ic H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyUsrPermsDefined H_valid); intros H_usrPermsDef.
unfold usrPermsDefined in *.
intros a p H_defPerm_a.
destruct_pre_stop H_pre.
destruct_post_stop H_post.
rewrite <- H_sim_1.
apply (H_usrPermsDef a p).
rewrite H_sim_3.
assumption.
Qed.

Lemma callPresUsrPermsDefined : forall (ic:iCmp)(sac:SACall), exec s (call ic sac) s' ->
                                                               usrPermsDefined s'.
Proof.
intros ic sac H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyUsrPermsDefined H_valid); intros H_usrPermsDef.
unfold usrPermsDefined in *.
intros a p H_defPerm_a.
rewrite <- H_post.
apply (H_usrPermsDef a p).
rewrite H_post.
assumption.
Qed.

Lemma readPresUsrPermsDefined : forall (ic:iCmp)(cp:CProvider)(u:uri), exec s (read ic cp u) s' ->
                                                                         usrPermsDefined s'.
Proof.
intros ic cp u H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyUsrPermsDefined H_valid); intros H_usrPermsDef.
unfold usrPermsDefined in *.
intros a p H_defPerm_a.
rewrite <- H_post.
apply (H_usrPermsDef a p).
rewrite H_post.
assumption.
Qed.

Lemma writePresUsrPermsDefined : forall (ic:iCmp)(cp:CProvider)(u:uri)(v:Val), exec s (write ic cp u v) s' ->
                                                                                 usrPermsDefined s'.
Proof.
intros ic cp u v H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyUsrPermsDefined H_valid); intros H_usrPermsDef.
unfold usrPermsDefined in *.
intros a p H_defPerm_a.
destruct_post_write H_post.
rewrite <- H_sim_1.
apply (H_usrPermsDef a p).
rewrite H_sim_3.
assumption.
Qed.

Lemma grantTPresUsrPermsDefined : forall (ic:iCmp)(cp:CProvider)(act:Activity)(u:uri)(pt:PType), exec s (grantT ic cp act u pt) s' ->
                                                                                                    usrPermsDefined s'.
Proof.
intros ic cp act u pt H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyUsrPermsDefined H_valid); intros H_usrPermsDef.
unfold usrPermsDefined in *.
intros a p H_defPerm_a.
destruct_post_grantT H_post.
rewrite <- H_sim_1.
apply (H_usrPermsDef a p).
rewrite H_sim_3.
assumption.
Qed.

Lemma grantPPresUsrPermsDefined : forall (ic:iCmp)(cp:CProvider)(a:App)(u:uri)(pt:PType), exec s (grantP ic cp a u pt) s' ->
                                                                                              usrPermsDefined s'.
Proof. 
intros ic cp a u pt H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyUsrPermsDefined H_valid); intros H_usrPermsDef.
unfold usrPermsDefined in *.
intros a' p H_defPerm_a'.
destruct_post_grantP H_post.
rewrite <- H_sim_1.
apply (H_usrPermsDef a' p).
rewrite H_sim_3.
assumption.
Qed.

Lemma revokePresUsrPermsDefined : forall (ic:iCmp)(cp:CProvider)(u:uri)(pt:PType), exec s (revoke ic cp u pt) s' ->
                                                                                     usrPermsDefined s'.
Proof.
intros ic cp u pt H_exec.
inversion H_exec as [H_valid H_pre H_post].
generalize (proyUsrPermsDefined H_valid); intros H_usrPermsDef.
unfold usrPermsDefined in *.
intros a p H_defPerm_a.
destruct_post_revoke H_post.
rewrite <- H_sim_1.
apply (H_usrPermsDef a p).
rewrite H_sim_3.
assumption.
Qed.

End AuxLemmas.

Section usrPermsDefinedInv.

Theorem usrPermsDefinedInv : forall (a:Action)(s s':State), exec s a s' -> usrPermsDefined s'.
Proof.
intros a s s'.
induction a.

exact (installPresUsrPermsDefined s s' a).
exact (uninstallPresUsrPermsDefined s s' a).
exact (startPresUsrPermsDefined s s' i c).
exact (stopPresUsrPermsDefined s s' i).
exact (callPresUsrPermsDefined s s' i s0).
exact (readPresUsrPermsDefined s s' i c u).
exact (writePresUsrPermsDefined s s' i c u v).
exact (grantTPresUsrPermsDefined s s' i c a u p).
exact (grantPPresUsrPermsDefined s s' i c  a u p).
exact (revokePresUsrPermsDefined s s' i c u p).
Qed.
End usrPermsDefinedInv.